#if !defined(AFX_COLOREDCONTROLS_H__20020226_0D16_1B29_088A_0080AD509054__INCLUDED_)
#define AFX_COLOREDCONTROLS_H__20020226_0D16_1B29_088A_0080AD509054__INCLUDED_

#pragma once

/////////////////////////////////////////////////////////////////////////////
// Colored Control - Windows Controls with colours
//
// Written by Bjarke Viksoe (bjarke@viksoe.dk)
// Copyright (c) 2002 Bjarke Viksoe.
//
// This file include the following controls:
//   CColoredDialog
//   CColoredStaticCtrl
//   CColoredEditCtrl
//   CColoredButtonCtrl
//   CColoredComboBoxCtrl
//   CColoredTabCtrl
//   CColoredListViewCtrl
//   CColoredTreeViewCtrl
//
// Add the following macro to the parent's message map:
//   REFLECT_NOTIFICATIONS()
//
// This code may be used in compiled form in any way you desire. This
// file may be redistributed by any means PROVIDING it is 
// not sold for profit without the authors written consent, and 
// providing that this notice and the authors name is included. 
//
// This file is provided "as is" with no expressed or implied warranty.
// The author accepts no liability if it causes any damage to you or your
// computer whatsoever. It's free, so don't hassle me about it.
//
// Beware of bugs.
//

#ifndef __cplusplus
  #error WTL requires C++ compilation (use a .cpp suffix)
#endif

#ifndef __ATLCTRLS_H__
  #error ColoredControls.h requires atlctrls.h to be included first
#endif




// Class used to create a font based on the font used by a specific window
class CWindowFont : public CFont  
{
public:
	// Font styles
	typedef enum tagEType
	{
		typeNormal		= 0x00,
		typeBold		= 0x01,
		typeItalic		= 0x02,
		typeUnderline		= 0x04,
		typeDoubleHeight	= 0x08,
	} EType;
public:
	CWindowFont() : CFont()
	{
	}
	
	/// hWnd	- The window to use for the base font
	/// nType	- Font style flags
	CWindowFont(HWND hWnd, int nType)
	{
		// We need a HWND
		ATLASSERT(hWnd != NULL);
		// Create the font
		Create(hWnd, nType);
	}
	
	virtual ~CWindowFont()
	{
	}
public:
	// Create the font
	// hWnd		- The window to use for the base font
	// nType	- Font style flags
	// return true on success
	bool Create(HWND hWnd, int nType)
	{
		// Be defensive
		ATLASSERT(hWnd != NULL);
		ATLASSERT(::IsWindow(hWnd) != FALSE);
		// Get the font the window is currently using
		HFONT hFont = (HFONT)::SendMessage(hWnd, WM_GETFONT, 0, 0);
		// Do we have a font?
		if (hFont == NULL)
			return false;
		
		CLogFont lf;		
		// Fill the LOGFONT
		if (::GetObject(hFont, sizeof(lf), &lf) == 0)
			return false;
		// Amend the LOGFONT members
		if (nType & typeBold)
			lf.lfWeight = FW_BOLD;
		if (nType & typeItalic)
			lf.lfItalic = TRUE;
		if (nType & typeUnderline)
			lf.lfUnderline = TRUE;
		if (nType & typeDoubleHeight)
			lf.lfHeight *= 2;
		
		// Create the new font
		return CreateFontIndirect(&lf) ? true : false;
	}
	
	// Create the font and apply to a nominate dialog control
	bool Apply(HWND hWnd, int nType)
	{
		// First create the font
		if (!Create(hWnd, nType))
			return false;
		// Apply to the nominated control
		CWindow wndControl = hWnd;
		ATLASSERT(wndControl != NULL);
		// Apply
		wndControl.SetFont(m_hFont);
		return true;
	}
};
/////////////////////////////////////////////////////////////////////////////
// Dialog with new colour or bitmap background

// To use this class: Derive from CColoredDialog<Base> and then
// chain the message map CHAIN_MSG_MAP(CColoredDialog<Base>)...
template< class T >
class CColoredDialog
{
public:
   CBrush m_brBack;

   // Operations

   BOOL SetBitmap(UINT nRes)
   {
      // Load the bitmap and create a new brush
      CBitmap bmBack;
      bmBack.LoadBitmap(nRes);
      if( !m_brBack.IsNull() ) m_brBack.DeleteObject();
      m_brBack.CreatePatternBrush(bmBack);
      // Repaint
      T* pT = static_cast<T*>(this);
      pT->Invalidate();
   }
   void SetColor(COLORREF clr)
   {
      // Create a new brush from the color
      if( !m_brBack.IsNull() ) m_brBack.DeleteObject();
      m_brBack.CreateSolidBrush(clr);
      // Repaint
      T* pT = static_cast<T*>(this);
      pT->Invalidate();
   }

   // Message map and handlers

   BEGIN_MSG_MAP(CColoredDialog)
      MESSAGE_HANDLER(WM_CTLCOLORDLG, OnCtlColorDlg)
   END_MSG_MAP()

   LRESULT OnCtlColorDlg(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM /*lParam*/, BOOL& /*bHandled*/)
   {
      T* pT = static_cast<T*>(this);
      if( m_brBack.IsNull() ) return pT->DefWindowProc();
      return (LRESULT) (HBRUSH) m_brBack;
   }
};


/////////////////////////////////////////////////////////////////////////////
// CColoredStaticCtrl - A Static control with new colours

template< class T, class TBase = CStatic, class TWinTraits = CControlWinTraits >
class ATL_NO_VTABLE CColoredStaticImpl : 
   public CWindowImpl< T, TBase, TWinTraits >
{
public:
   DECLARE_WND_SUPERCLASS(NULL, TBase::GetWndClassName())

   // Color codes for Static control
   COLORREF      m_clrBackground;
   COLORREF      m_clrNormalText;
   COLORREF      m_clrNormalBk;
   COLORREF      m_clrDisabledText;
   COLORREF      m_clrDisabledBk;
   // User-defined colours needs a brush
   CBrush        m_brBackground;
   CBrush        m_brNormalBk;
   CBrush        m_brDisabledBk;
   // We use a brush-handle when no custom colours have been assigned
   // because system-color brushes must not be destroyed...
   CBrushHandle  m_hbrNormalBk;
   CBrushHandle  m_hbrDisabledBk;

   // Operations
   CWindowFont m_fontBold;

   BOOL SubclassWindow(HWND hWnd)
   {
      ATLASSERT(m_hWnd==NULL);
      ATLASSERT(::IsWindow(hWnd));
#ifdef _DEBUG
      // Check class
      TCHAR szBuffer[16];
      if( ::GetClassName(m_hWnd, szBuffer, (sizeof(szBuffer)/sizeof(TCHAR))-1) ) {
         ATLASSERT(::lstrcmpi(szBuffer, TBase::GetWndClassName())==0);
      }
#endif
      BOOL bRet = CWindowImpl< T, TBase, TWinTraits >::SubclassWindow(hWnd);
      if( bRet ) _Init();
      return bRet;
   }

   void SetNormalColors(COLORREF clrText, COLORREF clrBack)
   {
      ATLASSERT(::IsWindow(m_hWnd));
      // Text 
      if( clrText == (COLORREF) -1 ) clrText = ::GetSysColor(COLOR_BTNTEXT);
      m_clrNormalText = clrText;
      // Background
      if( !m_brNormalBk.IsNull() ) m_brNormalBk.DeleteObject();
      if( clrBack == (COLORREF) -1 ) {
         m_clrNormalBk = ::GetSysColor(COLOR_BTNFACE);
         m_hbrNormalBk = ::GetSysColorBrush(COLOR_BTNFACE);
      }
      else {
         m_clrNormalBk = clrBack;
         m_brNormalBk.CreateSolidBrush(clrBack);
         m_hbrNormalBk = m_brNormalBk;
      }
      // Repaint
      Invalidate();
   }
   void SetDisabledColors(COLORREF clrText, COLORREF clrBack)
   {
      ATLASSERT(::IsWindow(m_hWnd));
      // Text 
      if( clrText == (COLORREF) -1 ) clrText = ::GetSysColor(COLOR_GRAYTEXT);
      m_clrDisabledText = clrText;
      // Background
      if( !m_brDisabledBk.IsNull() ) m_brDisabledBk.DeleteObject();
      if( clrBack == (COLORREF) -1 ) {
         m_clrDisabledBk = ::GetSysColor(COLOR_BTNFACE);
         m_hbrDisabledBk = ::GetSysColorBrush(COLOR_BTNFACE);
      }
      else {
         m_clrDisabledBk = clrBack;
         m_brDisabledBk.CreateSolidBrush(clrBack);
         m_hbrDisabledBk = m_brDisabledBk;
      }
      // Repaint
      Invalidate();
   }
   void SetBkColor(COLORREF clrBack)
   {
      ATLASSERT(::IsWindow(m_hWnd));
      // Background
      if( !m_brBackground.IsNull() ) m_brBackground.DeleteObject();
      m_clrBackground = clrBack;
      m_brBackground.CreateSolidBrush(clrBack);
      // Repaint
      Invalidate();
   }

   // Implementation

   virtual void _Init()
   {
      ATLASSERT(::IsWindow(m_hWnd));
      COLORREF clrNone = (COLORREF) -1;
      SetNormalColors( clrNone, clrNone );
      SetDisabledColors( clrNone, clrNone );
   }

   // Message map and handlers

   BEGIN_MSG_MAP(CColoredStaticImpl)
      MESSAGE_HANDLER(WM_CREATE, OnCreate)
      MESSAGE_HANDLER(OCM_CTLCOLORSTATIC, OnCtlColorStatic)
      MESSAGE_HANDLER(OCM_CTLCOLORDLG, OnCtlColorDlg)
      DEFAULT_REFLECTION_HANDLER()
   END_MSG_MAP()

   LRESULT OnCreate(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& /*bHandled*/)
   {
      LRESULT lRes = DefWindowProc(uMsg, wParam, lParam);
      _Init();
      return lRes;
   }

   LRESULT OnCtlColorStatic(UINT /*uMsg*/, WPARAM wParam, LPARAM /*lParam*/, BOOL& /*bHandled*/)
   {
      BOOL bEnabled = IsWindowEnabled();
      CDCHandle dc( (HDC) wParam );
      dc.SetBkMode(TRANSPARENT);
      dc.SetTextColor( bEnabled ? m_clrNormalText:m_clrDisabledText );
      dc.SetBkColor( bEnabled ? m_clrNormalBk : m_clrDisabledBk );
      return (LRESULT) (HBRUSH) ( bEnabled ? m_hbrNormalBk : m_hbrDisabledBk );
      dc.SetTextColor( m_clrNormalText );
      dc.SetBkColor( m_clrNormalBk );
      return (LRESULT) (HBRUSH) m_hbrNormalBk;
   }
   LRESULT OnCtlColorDlg(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM /*lParam*/, BOOL& /*bHandled*/)
   {
      T* pT = static_cast<T*>(this);
      if( m_brBackground.IsNull() ) return pT->DefWindowProc();
      return (LRESULT) (HBRUSH) m_brBackground;
   }
};

class CColoredStaticCtrl : public CColoredStaticImpl<CColoredStaticCtrl>
{
public:
   DECLARE_WND_SUPERCLASS(_T("WTL_ColoredStatic"), GetWndClassName())  
};

class CColoredStaticCtrlBold : public CColoredStaticImpl<CColoredStaticCtrlBold>
{
public:
   DECLARE_WND_SUPERCLASS(_T("WTL_ColoredStaticBold"), GetWndClassName())  
	void _Init()
   {
      ATLASSERT(::IsWindow(m_hWnd));
      COLORREF clrNone = (COLORREF) -1;
      SetNormalColors( clrNone, clrNone );
      SetDisabledColors( clrNone, clrNone );
	  m_fontBold.Apply(m_hWnd, CWindowFont::typeBold);
   }
};

class CColoredCheckboxCtrl : public CColoredStaticImpl<CColoredCheckboxCtrl, CButton>
{
public:
   DECLARE_WND_SUPERCLASS(_T("WTL_ColoredCheckbox"), GetWndClassName())  
};

class CColoredOptionCtrl : public CColoredStaticImpl<CColoredOptionCtrl, CButton>
{
public:
   DECLARE_WND_SUPERCLASS(_T("WTL_ColoredOption"), GetWndClassName())  
};



/////////////////////////////////////////////////////////////////////////////
// CColoredEditCtrl - An Edit control with new colours

template< class T, class TBase = CEdit, class TWinTraits = CControlWinTraits >
class ATL_NO_VTABLE CColoredEditImpl : 
   public CWindowImpl< T, TBase, TWinTraits >
{
public:
   DECLARE_WND_SUPERCLASS(NULL, TBase::GetWndClassName())
	BOOL m_bNULL;

/*    CColoredEditImpl()
	{
		
		//m_hPen = CreatePen(PS_SOLID, 1, ::GetSysColor(COLOR_3DDKSHADOW));
	}

	~CColoredEditImpl()
	{
		
		//DeleteObject(m_hPen);
	}
*//*	LRESULT OnNcPaint(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled)
	{
		RECT rect;
		GetWindowRect(&rect);
		CDCHandle dc = GetWindowDC();
		HPEN hOldPen = dc.SelectPen(m_hPen);
		rect.right -= rect.left;
		rect.bottom -= rect.top;
		rect.left = rect.top = 0;
		dc.Rectangle(&rect);
		dc.DrawEdge(&rect, BDR_SUNKENOUTER, BF_BOTTOMRIGHT);
		rect.top += 1; rect.bottom -= 1; rect.left += 1; rect.right -= 1;
		dc.SelectPen(hOldPen);
		ReleaseDC(dc);
		CString s;
		GetWindowText(s);
		SetWindowText(s);
		return 0;
	}
*/
	HPEN m_hPen;
   // Color codes for Edit control
   COLORREF      m_clrNormalText;
   COLORREF      m_clrNormalBk;
   COLORREF      m_clrDisabledText;
   COLORREF      m_clrDisabledBk;
   COLORREF      m_clrReadOnlyText;
   COLORREF      m_clrReadOnlyBk;
   // User-defined colours needs a brush
   CBrush        m_brNormalBk;
   CBrush        m_brDisabledBk;
   CBrush        m_brReadOnlyBk;
   // We use a brush-handle when no custom colours have been assigned
   // because system-color brushes must not be destroyed...
   CBrushHandle  m_hbrNormalBk;
   CBrushHandle  m_hbrDisabledBk;
   CBrushHandle  m_hbrReadOnlyBk;

   // Operations

   BOOL SubclassWindow(HWND hWnd)
   {
      ATLASSERT(m_hWnd==NULL);
      ATLASSERT(::IsWindow(hWnd));
#ifdef _DEBUG
      // Check class
      TCHAR szBuffer[16];
      if( ::GetClassName(m_hWnd, szBuffer, (sizeof(szBuffer)/sizeof(TCHAR))-1) ) {
         ATLASSERT(::lstrcmpi(szBuffer, TBase::GetWndClassName())==0);
      }
#endif
      BOOL bRet = CWindowImpl< T, TBase, TWinTraits >::SubclassWindow(hWnd);
      if( bRet ) _Init();
      return bRet;
   }

   void SetNormalColors(COLORREF clrText, COLORREF clrBack)
   {
      ATLASSERT(::IsWindow(m_hWnd));
      // Text 
      if( clrText == (COLORREF) -1 ) clrText = ::GetSysColor(COLOR_WINDOWTEXT);
      m_clrNormalText = clrText;
      // Background
      if( !m_brNormalBk.IsNull() ) m_brNormalBk.DeleteObject();
      if( clrBack == (COLORREF) -1 ) {
         m_clrNormalBk = ::GetSysColor(COLOR_WINDOW);
         m_hbrNormalBk = ::GetSysColorBrush(COLOR_WINDOW);
      }
      else {
         m_clrNormalBk = clrBack;
         m_brNormalBk.CreateSolidBrush(clrBack);
         m_hbrNormalBk = m_brNormalBk;
      }
      // Repaint
      Invalidate();
   }
   void SetDisabledColors(COLORREF clrText, COLORREF clrBack)
   {
      ATLASSERT(::IsWindow(m_hWnd));
      // Text 
      if( clrText == (COLORREF) -1 ) clrText = ::GetSysColor(COLOR_GRAYTEXT);
      m_clrDisabledText = clrText;
      // Background
      if( !m_brDisabledBk.IsNull() ) m_brDisabledBk.DeleteObject();
      if( clrBack == (COLORREF) -1 ) {
         m_clrDisabledBk = ::GetSysColor(COLOR_BTNFACE);
         m_hbrDisabledBk = ::GetSysColorBrush(COLOR_BTNFACE);
      }
      else {
         m_clrDisabledBk = clrBack;
         m_brDisabledBk.CreateSolidBrush(clrBack);
         m_hbrDisabledBk = m_brDisabledBk;
      }
      // Repaint
      Invalidate();
   }
   void SetReadOnlyColors(COLORREF clrText, COLORREF clrBack)
   {
      ATLASSERT(::IsWindow(m_hWnd));
      // Text 
      if( clrText == (COLORREF) -1 ) clrText = ::GetSysColor(COLOR_BTNTEXT);
      m_clrReadOnlyText = clrText;
      // Background
      if( !m_brReadOnlyBk.IsNull() ) m_brReadOnlyBk.DeleteObject();
      if( clrBack == (COLORREF) -1 ) {
         m_clrReadOnlyBk = ::GetSysColor(COLOR_BTNFACE);
         m_hbrReadOnlyBk = ::GetSysColorBrush(COLOR_BTNFACE);
      }
      else {
         m_clrReadOnlyBk = clrBack;
         m_brReadOnlyBk.CreateSolidBrush(clrBack);
         m_hbrReadOnlyBk = m_brReadOnlyBk;
      }
      // Repaint
      Invalidate();
   }

   // Implementation

   virtual void _Init()
   {
      ATLASSERT(::IsWindow(m_hWnd));
      COLORREF clrNone = (COLORREF) -1;
      SetNormalColors( clrNone, clrNone );
      SetDisabledColors( clrNone, clrNone );
      SetReadOnlyColors( clrNone, clrNone );
   }

   // Message map and handlers

   BEGIN_MSG_MAP(CColoredEditImpl)
      MESSAGE_HANDLER(WM_CREATE, OnCreate)
      MESSAGE_HANDLER(OCM_CTLCOLOREDIT, OnCtlColorEdit)
      MESSAGE_HANDLER(OCM_CTLCOLORMSGBOX, OnCtlColorEdit)
      MESSAGE_HANDLER(OCM_CTLCOLORSTATIC, OnCtlColorStatic)
//	  MESSAGE_HANDLER(WM_NCPAINT, OnNcPaint)
	  MESSAGE_HANDLER(WM_KILLFOCUS, OnKillFocus)
	  DEFAULT_REFLECTION_HANDLER()
   END_MSG_MAP()

   BOOL Check()
   {
		CString str;
		GetWindowText(str);
		if ((str.GetLength()<=0) && (m_bNULL == FALSE))
		{
			WrnMsg("  ", "   ", m_hWnd);
			SetFocus();
			return FALSE;
		}
		return TRUE;
   }
   LRESULT OnKillFocus(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM /*lParam*/, BOOL& /*bHandled*/)
   {
      LRESULT lRes = DefWindowProc();
      Invalidate();
	   //Check();
	   return 0;
   }
   LRESULT OnCreate(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& /*bHandled*/)
   {
      LRESULT lRes = DefWindowProc(uMsg, wParam, lParam);
      _Init();
      return lRes;
   }

   LRESULT OnCtlColorEdit(UINT /*uMsg*/, WPARAM wParam, LPARAM /*lParam*/, BOOL& /*bHandled*/)
   {
      CDCHandle dc( (HDC) wParam );
      dc.SetBkMode(TRANSPARENT);
      dc.SetTextColor( m_clrNormalText );
      dc.SetBkColor( m_clrNormalBk );
      return (LRESULT) (HBRUSH) m_hbrNormalBk;
   }

   LRESULT OnCtlColorStatic(UINT /*uMsg*/, WPARAM wParam, LPARAM /*lParam*/, BOOL& /*bHandled*/)
   {
      // Microsoft Q130952 explains why we also need this
      CDCHandle dc( (HDC) wParam );
      dc.SetBkMode(TRANSPARENT);
      if( IsWindowEnabled() ) {
         ATLASSERT(GetStyle() & ES_READONLY);
         dc.SetTextColor( m_clrReadOnlyText );
         dc.SetBkColor( m_clrReadOnlyBk );
         return (LRESULT) (HBRUSH) m_hbrReadOnlyBk;
      }
      else {
         dc.SetTextColor( m_clrDisabledText );
         dc.SetBkColor( m_clrDisabledBk );
         return (LRESULT) (HBRUSH) m_hbrDisabledBk;
      }
   }

};

class CColoredEditCtrl : public CColoredEditImpl<CColoredEditCtrl>
{
public:
   DECLARE_WND_SUPERCLASS(_T("WTL_ColoredEdit"), GetWndClassName())  
    CColoredEditCtrl()
	{
		m_bNULL = TRUE;
	}
};

class CColoredEditCtrlNN : public CColoredEditImpl<CColoredEditCtrlNN>
{
public:
   DECLARE_WND_SUPERCLASS(_T("WTL_ColoredEditNOTNULL"), GetWndClassName())  
   void _Init()
   {
		COLORREF clrBtnFace = ::GetSysColor(COLOR_BTNFACE);
		BYTE nRed = 0, nGreen = 0, nBlue = 0, nMax = 0;
		nRed = GetRValue(clrBtnFace);
		nGreen = GetGValue(clrBtnFace);
		nBlue = GetBValue(clrBtnFace);

		nMax = (nRed > nGreen) ? ((nRed > nBlue) ? nRed : nBlue) : ((nGreen > nBlue) ? nGreen : nBlue);
		const BYTE nMagicBackgroundOffset = (nMax > (0xFF - 35)) ? (BYTE)(0xFF - nMax) : (BYTE)35;
		if(nMax == 0)
		{
			nRed = (BYTE)(nRed + nMagicBackgroundOffset);
			nGreen = (BYTE)(nGreen + nMagicBackgroundOffset);
			nBlue = (BYTE)(nBlue + nMagicBackgroundOffset);
		}
		else
		{
			nRed = (BYTE)(nRed + (nMagicBackgroundOffset*(nRed/(double)nMax) + 0.5));
			nGreen = (BYTE)(nGreen + (nMagicBackgroundOffset*(nGreen/(double)nMax) + 0.5));
			nBlue = (BYTE)(nBlue + (nMagicBackgroundOffset*(nBlue/(double)nMax) + 0.5));
		}
      ATLASSERT(::IsWindow(m_hWnd));
      COLORREF clrNone = (COLORREF) -1;
      SetNormalColors( clrNone, RGB(nRed, nGreen, nBlue));
      SetDisabledColors( clrNone, RGB(nRed, nGreen, nBlue));
      SetReadOnlyColors( clrNone, RGB(nRed, nGreen, nBlue));
   }	
    CColoredEditCtrlNN()
	{
		m_bNULL = FALSE;
	}
};
/////////////////////////////////////////////////////////////////////////////
// CColoredButtonCtrl - The Button control with custom colours

template< class T, class TBase = CButton, class TWinTraits = CControlWinTraits >
class ATL_NO_VTABLE CColoredButtonImpl : 
   public CWindowImpl< T, TBase, TWinTraits >
{
public:
   DECLARE_WND_SUPERCLASS(NULL, TBase::GetWndClassName())

   // Color codes for Button control
   COLORREF      m_clrText;
   COLORREF      m_clrBackUp;
   COLORREF      m_clrBackDown;
   COLORREF      m_clrBackDisabled;
   // Image support
   CImageList    m_ImageList;
   UINT          m_nImage;
   UINT          m_nSelImage;

   // Operations

   BOOL SubclassWindow(HWND hWnd)
   {
      ATLASSERT(m_hWnd==NULL);
      ATLASSERT(::IsWindow(hWnd));
#ifdef _DEBUG
      // Check class
      TCHAR szBuffer[20];
      if( ::GetClassName(m_hWnd, szBuffer, (sizeof(szBuffer)/sizeof(TCHAR))-1) ) {
         ATLASSERT(::lstrcmpi(szBuffer, TBase::GetWndClassName())==0);
      }
#endif
      BOOL bRet = CWindowImpl< T, TBase, TWinTraits >::SubclassWindow(hWnd);
      if( bRet ) _Init();
      return bRet;
   }

   void SetTextColor(COLORREF clrText)
   {
      ATLASSERT(::IsWindow(m_hWnd));
      if( clrText == (COLORREF) -1 ) clrText = ::GetSysColor(COLOR_BTNTEXT);
      m_clrText = clrText;
      // Repaint
      Invalidate();
   }
   void SetBkColor(COLORREF clrUp, COLORREF clrDown=-1)
   {
      ATLASSERT(::IsWindow(m_hWnd));
      if( clrUp == (COLORREF) -1 ) clrUp = ::GetSysColor(COLOR_BTNFACE);
      if( clrDown == (COLORREF) -1 ) clrDown = clrUp;
      m_clrBackUp = clrUp;
      m_clrBackDown = clrDown;
      // Repaint
      Invalidate();
   }
   void SetDisabledColor(COLORREF clrDisabled)
   {
      ATLASSERT(::IsWindow(m_hWnd));
      if( clrDisabled == (COLORREF) -1 ) clrDisabled = ::GetSysColor(COLOR_BTNFACE);
      m_clrBackDisabled = clrDisabled;
      // Repaint
      Invalidate();
   }
   void SetImageList(HIMAGELIST hImageList)
   {
      ATLASSERT(::IsWindow(m_hWnd));
      m_ImageList = hImageList;
   }
   void SetImage(UINT nImage)
   {
      ATLASSERT(::IsWindow(m_hWnd));
      ATLASSERT(!m_ImageList.IsNull());
      m_nImage = nImage;
   }
   void SetSelImage(UINT nImage)
   {
      ATLASSERT(::IsWindow(m_hWnd));
      ATLASSERT(!m_ImageList.IsNull());
      m_nSelImage = nImage;
   }

   // Implementation

   void _Init()
   {
      ATLASSERT(::IsWindow(m_hWnd));

      // We need this style to prevent Windows from painting the button
      ModifyStyle(0, BS_OWNERDRAW);
      
      COLORREF clrNone = (COLORREF) -1;
      SetTextColor( clrNone );
      SetBkColor( clrNone, clrNone );
      SetDisabledColor( clrNone );
      m_nImage = m_nSelImage = (UINT)-1;
   }

   // Message map and handlers

   BEGIN_MSG_MAP(CColoredButtonImpl)
      MESSAGE_HANDLER(WM_CREATE, OnCreate)
      MESSAGE_HANDLER(WM_ERASEBKGND, OnEraseBackground)
      MESSAGE_HANDLER(WM_LBUTTONDBLCLK, OnLButtonDblClick)
      MESSAGE_HANDLER(OCM_DRAWITEM, OnDrawItem)
   END_MSG_MAP()

   LRESULT OnCreate(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& /*bHandled*/)
   {
      LRESULT lRes = DefWindowProc(uMsg, wParam, lParam);
      _Init();
      return lRes;
   }
   LRESULT OnEraseBackground(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM /*lParam*/, BOOL& /*bHandled*/)
   {
      return 1;   // no background needed
   }
   LRESULT OnLButtonDblClick(UINT /*uMsg*/, WPARAM wParam, LPARAM lParam, BOOL& /*bHandled*/)
   {
      return DefWindowProc(WM_LBUTTONDOWN, wParam, lParam);
   }   
   LRESULT OnDrawItem(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM lParam, BOOL& /*bHandled*/)
   {
      LPDRAWITEMSTRUCT lpDIS = (LPDRAWITEMSTRUCT) lParam;
      ATLASSERT(lpDIS->CtlType==ODT_BUTTON);
      CDCHandle dc = lpDIS->hDC;
      RECT rc = lpDIS->rcItem;
      DWORD dwStyle = GetStyle();
      bool bSelected = (lpDIS->itemState & ODS_SELECTED)!=0;
      bool bDisabled = (lpDIS->itemState & (ODS_DISABLED|ODS_GRAYED))!=0;

      COLORREF clrBack = bSelected ? m_clrBackDown : m_clrBackUp;
      if( bDisabled ) clrBack = m_clrBackDisabled;
      dc.FillSolidRect(&rc, clrBack);

      // Draw edge
      if( dwStyle & BS_FLAT ) {
         dc.DrawEdge(&rc, bSelected ? BDR_SUNKENOUTER : BDR_RAISEDINNER, BF_RECT);
      }
      else {
         dc.DrawEdge(&rc, bSelected ? EDGE_SUNKEN : EDGE_RAISED, BF_RECT);
      }
      ::InflateRect(&rc, -2 * ::GetSystemMetrics(SM_CXEDGE), -2 * ::GetSystemMetrics(SM_CYEDGE));

      // Draw focus rectangle
      if( lpDIS->itemState & ODS_FOCUS ) dc.DrawFocusRect(&rc);
      ::InflateRect(&rc, -1, -1);

      // Offset when button is selected
      if( bSelected ) ::OffsetRect(&rc, 1, 1);

      // Draw image
      UINT nImage = m_nImage;
      if( bSelected && m_nSelImage!=-1 ) nImage = m_nSelImage;
      if( !m_ImageList.IsNull() && nImage!=-1 ) {
         POINT pt = { rc.left, rc.top-1 };
         SIZE sizeIcon;
         m_ImageList.GetIconSize(sizeIcon);
         if( bDisabled ) {
            //m_ImageList.DrawEx(nImage, dc, pt.x, pt.y, sizeIcon.cx, sizeIcon.cy, CLR_NONE, ::GetSysColor(COLOR_GRAYTEXT), ILD_BLEND50 | ILD_TRANSPARENT);
            HICON hIcon = m_ImageList.GetIcon(nImage);
            dc.DrawState(pt, sizeIcon, hIcon, DST_ICON | DSS_DISABLED);
            ::DestroyIcon(hIcon);
         }
         else {
            m_ImageList.Draw(dc, nImage, pt, bDisabled ? ILD_BLEND25 : ILD_TRANSPARENT);
         }
         rc.left += sizeIcon.cx + 3;
      }

      // Draw text
      UINT nLen = GetWindowTextLength();
      LPTSTR pstr = (LPTSTR) _alloca( (nLen+1)*sizeof(TCHAR) );
      GetWindowText(pstr, nLen+1);
      UINT uFlags = 0;
      if( dwStyle & BS_LEFT ) uFlags |= DT_LEFT;
      else if( dwStyle & BS_RIGHT ) uFlags |= DT_RIGHT;
      else if( dwStyle & BS_CENTER ) uFlags |= DT_CENTER;
      else uFlags |= DT_CENTER;
      if( dwStyle & BS_TOP ) uFlags |= DT_TOP;
      else if( dwStyle & BS_BOTTOM ) uFlags |= DT_BOTTOM;
      else if( dwStyle & BS_VCENTER ) uFlags |= DT_VCENTER;
      else uFlags |= DT_VCENTER;
      if( (dwStyle & BS_MULTILINE)==0 ) uFlags |= DT_SINGLELINE;
      dc.SetBkMode(TRANSPARENT);
      dc.SetTextColor(bDisabled ? ::GetSysColor(COLOR_GRAYTEXT) : m_clrText);
      dc.DrawText(pstr, nLen, &rc, uFlags);

      return TRUE;
   }
};

class CColoredButtonCtrl : public CColoredButtonImpl<CColoredButtonCtrl>
{
public:
   DECLARE_WND_SUPERCLASS(_T("WTL_ColoredButton"), GetWndClassName())  
};

class CColoredButtonCtrlExt : public CColoredButtonImpl<CColoredButtonCtrlExt>
{
public:
   DECLARE_WND_SUPERCLASS(_T("WTL_ColoredButtonEXT"), GetWndClassName())  
   void _Init()
   {
      ATLASSERT(::IsWindow(m_hWnd));
      SetTextColor(RGB(255,255,255));
	  SetBkColor(RGB(100,100,100));
   }	
};

/////////////////////////////////////////////////////////////////////////////
// CColoredComboBoxCtrl - The ComboBox control with custom colours

template< class T, class TBase = CComboBox, class TWinTraits = CControlWinTraits >
class ATL_NO_VTABLE CColoredComboBoxImpl : 
   public CWindowImpl< T, TBase, TWinTraits >
{
public:
   
   DECLARE_WND_SUPERCLASS(NULL, TBase::GetWndClassName())

   // Color codes for ComboBox control
   COLORREF      m_clrListText;
   COLORREF      m_clrListBk;
   COLORREF      m_clrSelectedListText;
   COLORREF      m_clrSelectedListBk;
   COLORREF      m_clrDisabledText;
   COLORREF      m_clrDisabledBk;
   COLORREF      m_clrEditText;
   COLORREF      m_clrEditBk;
   // User-defined colours needs a brush
   CBrush        m_brListBk;
   CBrush        m_brSelectedListBk;
   CBrush        m_brDisabledBk;
   CBrush        m_brEditBk;
   // We use a brush-handle when no custom colours have been assigned
   // because system-color brushes must not be destroyed...
   CBrushHandle  m_hbrListBk;
   CBrushHandle  m_hbrSelectedListBk;
   CBrushHandle  m_hbrDisabledBk;
   CBrushHandle  m_hbrEditBk;

   BOOL m_bNULL;
   // Message map and handlers

   BEGIN_MSG_MAP(CColoredComboBoxImpl)
      REFLECTED_COMMAND_CODE_HANDLER(CBN_EDITUPDATE, OnEditUpdate)
      REFLECTED_COMMAND_CODE_HANDLER(CBN_KILLFOCUS, OnFocusChange);
      MESSAGE_HANDLER(WM_CREATE, OnCreate)
      MESSAGE_HANDLER(WM_CTLCOLOREDIT, OnCtlColorEdit)
      MESSAGE_HANDLER(OCM_CTLCOLOREDIT, OnCtlColorEdit)
      MESSAGE_HANDLER(WM_CTLCOLORMSGBOX, OnCtlColorEdit)
      MESSAGE_HANDLER(WM_CTLCOLORLISTBOX, OnCtlColorListBox)
      MESSAGE_HANDLER(OCM_CTLCOLORSTATIC, OnCtlColorStatic)
      DEFAULT_REFLECTION_HANDLER()
   END_MSG_MAP()

   BOOL Check()
   {
		if ((GetCount()>0) && (m_bNULL == FALSE))
		{
			CString str;
			GetWindowText(str);
			if (str.GetLength()<=0) 
			{
				WrnMsg("  ", "   ", m_hWnd);
				SetFocus();
				return FALSE;
			}
			if (FindString(0,str)<0)
			{
				WrnMsg("  ", "   ", m_hWnd);
				SetFocus();
				return FALSE;
			}
		}
		return TRUE;
   }
	LRESULT OnFocusChange(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
   {
      LRESULT lRes = DefWindowProc();
      Invalidate();
// 		Check();
     return lRes;
   }
	LRESULT OnEditUpdate(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
	{
		CString str;
		GetWindowText(str);
		int nLength = str.GetLength();

		DWORD dwCurSel = GetEditSel();
		WORD dStart = LOWORD(dwCurSel);
		WORD dEnd   = HIWORD(dwCurSel);

		if (SelectString(-1, str) == CB_ERR)
		{
			SetWindowText(str);		// No text selected, so restore what was there before
			if (dwCurSel != CB_ERR)
				SetEditSel(dStart, dEnd);	//restore cursor postion
		}

		// Set the text selection as the additional text that we have added
		if (dEnd < nLength && dwCurSel != CB_ERR)
			SetEditSel(dStart, dEnd);
		else
			SetEditSel(nLength, -1);
		return 0;
	}
   // Operations
   BOOL SubclassWindow(HWND hWnd)
   {
      ATLASSERT(m_hWnd==NULL);
      ATLASSERT(::IsWindow(hWnd));
#ifdef _DEBUG
      // Check class
      TCHAR szBuffer[16];
      if( ::GetClassName(m_hWnd, szBuffer, (sizeof(szBuffer)/sizeof(TCHAR))-1) ) {
         ATLASSERT(::lstrcmpi(szBuffer, TBase::GetWndClassName())==0);
      }
#endif
      BOOL bRet = CWindowImpl< T, TBase, TWinTraits >::SubclassWindow(hWnd);
      if( bRet ) _Init();
      return bRet;
   }

   void SetListColors(COLORREF clrText, COLORREF clrBack)
   {
      ATLASSERT(::IsWindow(m_hWnd));
      // Text 
      if( clrText == (COLORREF) -1 ) clrText = ::GetSysColor(COLOR_WINDOWTEXT);
      m_clrListText = clrText;
      // Background
      if( !m_brListBk.IsNull() ) m_brListBk.DeleteObject();
      if( clrBack == (COLORREF) -1 ) {
         m_clrListBk = ::GetSysColor(COLOR_WINDOW);
         m_hbrListBk = ::GetSysColorBrush(COLOR_WINDOW);
      }
      else {
         m_clrListBk = clrBack;
         m_brListBk.CreateSolidBrush(clrBack);
         m_hbrListBk = m_brListBk;
      }
      // Repaint
      Invalidate();
   }
   void SetDisabledColors(COLORREF clrText, COLORREF clrBack)
   {
      ATLASSERT(::IsWindow(m_hWnd));
      // Text 
      if( clrText == (COLORREF) -1 ) clrText = ::GetSysColor(COLOR_GRAYTEXT);
      m_clrDisabledText = clrText;
      // Background
      if( !m_brDisabledBk.IsNull() ) m_brDisabledBk.DeleteObject();
      if( clrBack == (COLORREF) -1 ) {
         m_clrDisabledBk = ::GetSysColor(COLOR_BTNFACE);
         m_hbrDisabledBk = ::GetSysColorBrush(COLOR_BTNFACE);
      }
      else {
         m_clrDisabledBk = clrBack;
         m_brDisabledBk.CreateSolidBrush(clrBack);
         m_hbrDisabledBk = m_brDisabledBk;
      }
      // Repaint
      Invalidate();
   }
   void SetEditColors(COLORREF clrText, COLORREF clrBack)
   {
      ATLASSERT(::IsWindow(m_hWnd));
      // Text 
      if( clrText == (COLORREF) -1 ) clrText = ::GetSysColor(COLOR_BTNTEXT);
      m_clrEditText = clrText;
      // Background
      if( !m_brEditBk.IsNull() ) m_brEditBk.DeleteObject();
      if( clrBack == (COLORREF) -1 ) {
         m_clrEditBk = ::GetSysColor(COLOR_WINDOW);
         m_hbrEditBk = ::GetSysColorBrush(COLOR_WINDOW);
      }
      else {
         m_clrEditBk = clrBack;
         m_brEditBk.CreateSolidBrush(clrBack);
         m_hbrEditBk = m_brEditBk;
      }
      // Repaint
      Invalidate();
   }

   // Implementation

   virtual void _Init()
   {
      ATLASSERT(::IsWindow(m_hWnd));
      COLORREF clrNone = (COLORREF) -1;
      SetListColors( clrNone, clrNone );
      SetDisabledColors( clrNone, clrNone );
      SetEditColors( clrNone, clrNone );
   }

   LRESULT OnCreate(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& /*bHandled*/)
   {
      LRESULT lRes = DefWindowProc(uMsg, wParam, lParam);
      _Init();
      return lRes;
   }

   LRESULT OnCtlColorEdit(UINT /*uMsg*/, WPARAM wParam, LPARAM /*lParam*/, BOOL& /*bHandled*/)
   {
      CDCHandle dc( (HDC) wParam );
      dc.SetBkMode(TRANSPARENT);
      dc.SetTextColor( m_clrEditText );
      dc.SetBkColor( m_clrEditBk );
      return (LRESULT) (HBRUSH) m_hbrEditBk;
   }
   LRESULT OnCtlColorListBox(UINT /*uMsg*/, WPARAM wParam, LPARAM /*lParam*/, BOOL& /*bHandled*/)
   {
      CDCHandle dc( (HDC) wParam );
      dc.SetBkMode(TRANSPARENT);
      dc.SetTextColor( m_clrListText );
      dc.SetBkColor( m_clrListBk );
      return (LRESULT) (HBRUSH) m_hbrListBk;
   }
   LRESULT OnCtlColorStatic(UINT /*uMsg*/, WPARAM wParam, LPARAM /*lParam*/, BOOL& /*bHandled*/)
   {
      CDCHandle dc( (HDC) wParam );
      dc.SetBkMode(TRANSPARENT);
      if( IsWindowEnabled() ) {
         dc.SetTextColor( m_clrEditText );
         dc.SetBkColor( m_clrEditBk );
         return (LRESULT) (HBRUSH) m_hbrEditBk;
      }
      else {
         dc.SetTextColor( m_clrDisabledText );
         dc.SetBkColor( m_clrDisabledBk );
         return (LRESULT) (HBRUSH) m_hbrDisabledBk;
      }
   }
};

class CColoredComboBoxCtrl : public CColoredComboBoxImpl<CColoredComboBoxCtrl>
{
public:
   DECLARE_WND_SUPERCLASS(_T("WTL_ColoredComboBox"), GetWndClassName())  
	CColoredComboBoxCtrl()
	{
		m_bNULL = TRUE;
	}
};
class CColoredComboBoxCtrlNN : public CColoredComboBoxImpl<CColoredComboBoxCtrlNN>
{
public:
   DECLARE_WND_SUPERCLASS(_T("WTL_ColoredComboBoxNOTNULL"), GetWndClassName())  
	void _Init()
   {
		COLORREF clrBtnFace = ::GetSysColor(COLOR_BTNFACE);
		BYTE nRed = 0, nGreen = 0, nBlue = 0, nMax = 0;
		nRed = GetRValue(clrBtnFace);
		nGreen = GetGValue(clrBtnFace);
		nBlue = GetBValue(clrBtnFace);

		nMax = (nRed > nGreen) ? ((nRed > nBlue) ? nRed : nBlue) : ((nGreen > nBlue) ? nGreen : nBlue);
		const BYTE nMagicBackgroundOffset = (nMax > (0xFF - 35)) ? (BYTE)(0xFF - nMax) : (BYTE)35;
		if(nMax == 0)
		{
			nRed = (BYTE)(nRed + nMagicBackgroundOffset);
			nGreen = (BYTE)(nGreen + nMagicBackgroundOffset);
			nBlue = (BYTE)(nBlue + nMagicBackgroundOffset);
		}
		else
		{
			nRed = (BYTE)(nRed + (nMagicBackgroundOffset*(nRed/(double)nMax) + 0.5));
			nGreen = (BYTE)(nGreen + (nMagicBackgroundOffset*(nGreen/(double)nMax) + 0.5));
			nBlue = (BYTE)(nBlue + (nMagicBackgroundOffset*(nBlue/(double)nMax) + 0.5));
		}
      ATLASSERT(::IsWindow(m_hWnd));
      COLORREF clrNone = (COLORREF) -1;
      SetListColors( clrNone, RGB(nRed, nGreen, nBlue));
      SetDisabledColors( clrNone, RGB(nRed, nGreen, nBlue));
      SetEditColors( clrNone, RGB(nRed, nGreen, nBlue));
   }	
   CColoredComboBoxCtrlNN()
	{
		m_bNULL = FALSE;
	}
};


/////////////////////////////////////////////////////////////////////////////
// CColoredTabCtrl - A Tab control with tab colours

template< class T, class TBase = CTabCtrl, class TWinTraits = CControlWinTraits >
class ATL_NO_VTABLE CColoredTabImpl : 
   public CWindowImpl< T, TBase, TWinTraits >
{
public:
   DECLARE_WND_SUPERCLASS(NULL, TBase::GetWndClassName())

   // Color codes for Tab control
   COLORREF m_clrBackground;
   COLORREF m_clrNormalText;
   COLORREF m_clrNormalBk;
   COLORREF m_clrDisabledText;
   COLORREF m_clrDisabledBk;
   COLORREF m_clrInactiveText;
   COLORREF m_clrInactiveBk;
   COLORREF m_clrHighlightText;
   COLORREF m_clrHighlightBk;
   HFONT    m_hInactiveFont;

   // Operations

   BOOL SubclassWindow(HWND hWnd)
   {
      ATLASSERT(m_hWnd==NULL);
      ATLASSERT(::IsWindow(hWnd));
#ifdef _DEBUG
      // Check class
      TCHAR szBuffer[16];
      if( ::GetClassName(m_hWnd, szBuffer, (sizeof(szBuffer)/sizeof(TCHAR))-1) ) {
         ATLASSERT(::lstrcmpi(szBuffer, TBase::GetWndClassName())==0);
      }
#endif
      BOOL bRet = CWindowImpl< T, TBase, TWinTraits >::SubclassWindow(hWnd);
      if( bRet ) _Init();
      return bRet;
   }

   void SetBackgroundColor(COLORREF clrBack)
   {
      if( clrBack == (COLORREF) -1 ) clrBack = ::GetSysColor(COLOR_BTNFACE);
      m_clrBackground = clrBack;
      // Repaint
      Invalidate();
   }
   void SetNormalColors(COLORREF clrText, COLORREF clrBack)
   {
      ATLASSERT(::IsWindow(m_hWnd));
      // Text 
      if( clrText == (COLORREF) -1 ) clrText = ::GetSysColor(COLOR_WINDOWTEXT);
      m_clrNormalText = clrText;
      // Background
      if( clrBack == (COLORREF) -1 ) clrBack = ::GetSysColor(COLOR_BTNFACE);
      m_clrNormalBk = clrBack;
      // Repaint
      Invalidate();
   }
   void SetDisabledColors(COLORREF clrText, COLORREF clrBack)
   {
      ATLASSERT(::IsWindow(m_hWnd));
      // Text 
      if( clrText == (COLORREF) -1 ) clrText = ::GetSysColor(COLOR_GRAYTEXT);
      m_clrDisabledText = clrText;
      // Background
      if( clrBack == (COLORREF) -1 ) clrBack = ::GetSysColor(COLOR_BTNFACE);
      m_clrDisabledBk = clrBack;
      // Repaint
      Invalidate();
   }
   void SetInactiveColors(COLORREF clrText, COLORREF clrBack)
   {
      ATLASSERT(::IsWindow(m_hWnd));
      // Text 
      if( clrText == (COLORREF) -1 ) clrText = ::GetSysColor(COLOR_BTNTEXT);
      m_clrInactiveText = clrText;
      // Background
      if( clrBack == (COLORREF) -1 ) clrBack = ::GetSysColor(COLOR_BTNFACE);
      m_clrInactiveBk = clrBack;
      // Repaint
      Invalidate();
   }
   void SetHighlightColors(COLORREF clrText, COLORREF clrBack)
   {
      ATLASSERT(::IsWindow(m_hWnd));
      // Text 
#if(WINVER >= 0x0500)
      if( clrText == (COLORREF) -1 ) clrText = ::GetSysColor(COLOR_HOTLIGHT);
#else
      if( clrText == (COLORREF) -1 ) clrText = ::GetSysColor(COLOR_HIGHLIGHT);
#endif
      m_clrHighlightText = clrText;
      // Background
      if( clrBack == (COLORREF) -1 ) clrBack = ::GetSysColor(COLOR_BTNFACE);
      m_clrHighlightBk = clrBack;
      // Repaint
      Invalidate();
   }
   void SetInactiveFont(HFONT hFont)
   {
      m_hInactiveFont = hFont;
      // Repaint
      Invalidate();
   }

   // Implementation

   void _Init()
   {
      ATLASSERT(::IsWindow(m_hWnd));
      ATLASSERT((GetStyle() & (TCS_BUTTONS|TCS_VERTICAL))==0);

      // We're drawing the text...
      ModifyStyle(0, TCS_OWNERDRAWFIXED);

      m_hInactiveFont = NULL;
      COLORREF clrNone = (COLORREF) -1;
      SetBackgroundColor( clrNone );
      SetNormalColors( clrNone, clrNone );
      SetDisabledColors( clrNone, clrNone );
      SetInactiveColors( clrNone, clrNone );
      SetHighlightColors( clrNone, clrNone );
   }

   // Message map and handlers

   BEGIN_MSG_MAP(CColoredTabImpl)
      MESSAGE_HANDLER(WM_CREATE, OnCreate)
      MESSAGE_HANDLER(WM_PAINT, OnPaint)
      MESSAGE_HANDLER(WM_PRINTCLIENT, OnPaint)
      MESSAGE_HANDLER(OCM_DRAWITEM, OnDrawItem)
      DEFAULT_REFLECTION_HANDLER()
   END_MSG_MAP()

   LRESULT OnCreate(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& /*bHandled*/)
   {
      LRESULT lRes = DefWindowProc(uMsg, wParam, lParam);
      _Init();
      return lRes;
   }

   LRESULT OnDrawItem(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM lParam, BOOL& /*bHandled*/)
   {
      LPDRAWITEMSTRUCT lpDIS = (LPDRAWITEMSTRUCT) lParam;
      ATLASSERT(lpDIS->CtlType==ODT_TAB);
      ATLASSERT(lpDIS->itemAction & ODA_DRAWENTIRE);
      CDCHandle dc = lpDIS->hDC;
      RECT rc = lpDIS->rcItem;

      DWORD dwStyle = GetStyle();
      HFONT hFont = GetFont();

      TCHAR szText[128];
      TCITEM itm = { 0 };
      itm.mask = TCIF_TEXT | TCIF_IMAGE | TCIF_STATE;
      itm.pszText = szText;
      itm.cchTextMax = sizeof(szText)/sizeof(TCHAR);
      itm.dwStateMask = (DWORD) -1;
      GetItem(lpDIS->itemID, &itm);

      COLORREF clrText;
      COLORREF clrBack;
      int cyOffset = 0;
      if( lpDIS->itemState & ODS_DISABLED ) {
         clrText = m_clrDisabledText;
         clrBack = m_clrDisabledBk;
      }
      else if( itm.dwState & TCIS_HIGHLIGHTED ) {
         clrText = m_clrHighlightText;
         clrBack = m_clrHighlightBk;
      }
      else if( lpDIS->itemState & ODS_SELECTED ) {
         clrText = m_clrNormalText;
         clrBack = m_clrNormalBk;
         cyOffset = 1;
      }
      else {
         clrText = m_clrInactiveText;
         clrBack = m_clrInactiveBk;
         if( m_hInactiveFont!=NULL ) hFont = m_hInactiveFont;
      }

      dc.FillSolidRect(&rc, clrBack);

      if( itm.iImage != -1 ) {
         CImageList iml = GetImageList();
         int cxImage, cyImage;
         iml.GetIconSize(cxImage, cyImage);
         POINT pt = { rc.left + 5, rc.top + 1 + ((rc.bottom-rc.top)/2-(cyImage)/2) };
         iml.Draw(dc, itm.iImage, pt, ILD_TRANSPARENT );
         rc.left += cxImage;
      }

      HFONT hOldFont = dc.SelectFont(hFont);
      dc.SetBkColor(clrBack),
      dc.SetTextColor(clrText);
      dc.SetBkMode(TRANSPARENT);
      UINT dwFlags = DT_VCENTER | DT_SINGLELINE | DT_NOCLIP;
      if( dwStyle & TCS_FORCELABELLEFT ) {
         dwFlags |= DT_LEFT; 
         rc.left += 3;
      }
      else {
         dwFlags |= DT_CENTER;
      }
      dc.DrawText(itm.pszText, -1, &rc, dwFlags);
      dc.SelectFont(hOldFont);

      return TRUE;
   }

   LRESULT OnPaint(UINT /*uMsg*/, WPARAM wParam, LPARAM /*lParam*/, BOOL& bHandled)
   {
      T* pT = static_cast<T*>(this);
      if( wParam != NULL ) {
         RECT rc;
         GetClientRect(&rc);
         pT->DoPaint( (HDC) wParam );
         // Let the tab-control paint itself
         bHandled = FALSE;
      }
      else {
         PAINTSTRUCT m_ps;
         HDC hDC = ::BeginPaint(m_hWnd, &m_ps);
         pT->DoPaint(hDC);
         // Let the tab-control paint itself
         DefWindowProc(WM_PRINTCLIENT, (WPARAM) hDC, PRF_CLIENT);
         ::EndPaint(m_hWnd, &m_ps);
      }
      return 0;
   }

   void DoPaint(CDCHandle dc)
   {
      // Calculate the upper box, so we can paint a
      // different background colour. We need to do the calculation
      // because of a possible multi-line tab-control...
      RECT rcWin;
      GetWindowRect(&rcWin);
      RECT rcClient = rcWin;
      AdjustRect(FALSE, &rcClient);
      RECT rcTop = { 0, 0, rcWin.right-rcWin.left, rcClient.top-rcWin.top };
      dc.FillSolidRect(&rcTop, m_clrBackground);
      // TODO: If you want to paint the tab client-area a different
      //       colour, this is the place to do it...
   }
};

class CColoredTabCtrl : public CColoredTabImpl<CColoredTabCtrl>
{
public:
   DECLARE_WND_SUPERCLASS(_T("WTL_ColoredTab"), GetWndClassName())  
};


/////////////////////////////////////////////////////////////////////////////
// CColoredListViewCtrl - A ListView control with individual item colours

template< class T, class TBase = CListViewCtrl, class TWinTraits = CControlWinTraits >
class ATL_NO_VTABLE CColoredListViewImpl : 
   public CWindowImpl< T, TBase, TWinTraits >,
   public CCustomDraw< T >
{
public:
   DECLARE_WND_SUPERCLASS(NULL, TBase::GetWndClassName())

   typedef struct {
      COLORREF clrText;
      COLORREF clrBackground;
      HFONT    hFont;
      LPARAM   lParam;
   } LVCOLORPARAM;
   int m_nColumns;

   // Operations

   BOOL SubclassWindow(HWND hWnd)
   {
      ATLASSERT(m_hWnd==NULL);
      ATLASSERT(::IsWindow(hWnd));
#ifdef _DEBUG
      // Check class
      TCHAR szBuffer[16];
      if( ::GetClassName(m_hWnd, szBuffer, (sizeof(szBuffer)/sizeof(TCHAR))-1) ) {
         ATLASSERT(::lstrcmpi(szBuffer, TBase::GetWndClassName())==0);
      }
#endif
      BOOL bRet = CWindowImpl< T, TBase, TWinTraits >::SubclassWindow(hWnd);
      if( bRet ) _Init();
      return bRet;
   }

   int InsertItem(UINT nMask, int nItem, LPCTSTR lpszItem, UINT nState, UINT nStateMask, int nImage=-1, LPARAM lParam=0)
   {
      ATLASSERT(::IsWindow(m_hWnd));
      // You must have initialized columns before calling this!
      // And you are not allowed to add columns once the list is populated!
      CHeaderCtrl ctrlHeader = GetHeader();
      if( m_nColumns==0 ) m_nColumns = ctrlHeader.GetItemCount();
      ATLASSERT(m_nColumns > 0);
      ATLASSERT(ctrlHeader.GetItemCount()==m_nColumns);
      // Create a place-holder of property controls for each subitem...
      LVCOLORPARAM* pParams;
      ATLTRY( pParams = new LVCOLORPARAM[m_nColumns] );
      ATLASSERT(pParams);
      if( pParams==NULL ) return -1;
      ::FillMemory(pParams, sizeof(LVCOLORPARAM) * m_nColumns, -1);
      // Finally create the listview item itself...
      if( nItem==-1 ) nItem = GetItemCount();
      nMask |= LVIF_PARAM;
      pParams[0].lParam = lParam;
      return TBase::InsertItem(nMask, nItem, lpszItem, nState, nStateMask, nImage, (LPARAM) pParams);
   }
   int InsertItem(int nItem, LPCTSTR lpszItem, int nImage)
   {
      ATLASSERT(::IsWindow(m_hWnd));
      return InsertItem(LVIF_TEXT|LVIF_IMAGE, nItem, lpszItem, 0, 0, nImage, 0);
   }
   DWORD_PTR GetItemData(int nItem) const
   {
      ATLASSERT(::IsWindow(m_hWnd));
      LVITEM lvi = { 0 };
      lvi.iItem = nItem;
      lvi.mask = LVIF_PARAM;
      if( GetItem(&lvi)==-1 ) return 0;
      LVCOLORPARAM* pParams = (LVCOLORPARAM*) lvi.lParam;
      return (DWORD_PTR) pParams[0].lParam;
   }
   BOOL SetItemColors(int nItem, int nSubItem, COLORREF clrText, COLORREF clrBk)
   {
      ATLASSERT(::IsWindow(m_hWnd));
      LVITEM lvi = { 0 };
      lvi.iItem = nItem;
      lvi.mask = LVIF_PARAM;
      if( GetItem(&lvi)==-1 ) return FALSE;
      LVCOLORPARAM* pParams = (LVCOLORPARAM*) lvi.lParam;
      ATLASSERT(nSubItem>=0 && nSubItem<GetHeader().GetItemCount());
      pParams[ nSubItem ].clrText = clrText;
      pParams[ nSubItem ].clrBackground = clrBk;
      // Repaint
      RedrawItems(nItem, nItem);
      return TRUE;
   }

   // Unsupported ListView methods

   BOOL SetItemData(int /*nItem*/, DWORD_PTR /*dwData*/)
   {
      ATLASSERT(false); // not supported
      return FALSE;
   }
   int AddItem(int /*nItem*/, int /*nSubItem*/, LPCTSTR /*strItem*/, int /*nImageIndex = -1*/)
   {
      ATLASSERT(false); // not supported
      return -1;
   }

   // Implementation

   void _Init()
   {
      ATLASSERT(::IsWindow(m_hWnd));
      m_nColumns = 0;
   }

   // Message map and handlers

   BEGIN_MSG_MAP(CColoredListViewImpl)
      MESSAGE_HANDLER(WM_CREATE, OnCreate)
      REFLECTED_NOTIFY_CODE_HANDLER(LVN_DELETEITEM, OnDeleteItem)
      CHAIN_MSG_MAP_ALT( CCustomDraw< T >, 1 )
      // Because WTL 3.1 does not support subitem custom drawing.
      // Should be forward-compatible with new versions...
      REFLECTED_NOTIFY_CODE_HANDLER(NM_CUSTOMDRAW, OnNotifyCustomDraw)
      DEFAULT_REFLECTION_HANDLER()
   END_MSG_MAP()

   LRESULT OnCreate(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& /*bHandled*/)
   {
      LRESULT lRes = DefWindowProc(uMsg, wParam, lParam);
      _Init();
      return lRes;
   }
   LRESULT OnDeleteItem(int /*idCtrl*/, LPNMHDR pnmh, BOOL& /*bHandled*/)
   {
      ATLASSERT(m_nColumns>0);
      LPNMLISTVIEW pnmlv = (LPNMLISTVIEW) pnmh;
      ATLASSERT(pnmlv->lParam);
      LVCOLORPARAM* pParams = reinterpret_cast<LVCOLORPARAM*>(pnmlv->lParam);
      delete [] pParams;
      return 0;
   }
   LRESULT OnNotifyCustomDraw(int idCtrl, LPNMHDR pnmh, BOOL& bHandled)
   {
      T* pT = static_cast<T*>(this);
      pT->SetMsgHandled(FALSE);
      LPNMCUSTOMDRAW lpNMCustomDraw = (LPNMCUSTOMDRAW) pnmh;
      DWORD dwRet = 0;
      switch( lpNMCustomDraw->dwDrawStage ) {
      case CDDS_ITEMPREPAINT | CDDS_SUBITEM:
         dwRet = pT->OnSubItemPrePaint(idCtrl, lpNMCustomDraw);
         pT->SetMsgHandled(TRUE);
         return dwRet;
      }
      bHandled = FALSE;
      return dwRet;
   }

   // Custom painting

   DWORD OnPrePaint(int /*idCtrl*/, LPNMCUSTOMDRAW /*lpNMCustomDraw*/)
   {
      return CDRF_NOTIFYITEMDRAW;   // We need per-item notifications
   }
   DWORD OnItemPrePaint(int /*idCtrl*/, LPNMCUSTOMDRAW /*lpNMCustomDraw*/)
   {
      return CDRF_NOTIFYSUBITEMDRAW; // We need per-subitem notifications
   }
   DWORD OnSubItemPrePaint(int /*idCtrl*/, LPNMCUSTOMDRAW lpNMCustomDraw)
   {
      LPNMLVCUSTOMDRAW lpNMLVCD = (LPNMLVCUSTOMDRAW) lpNMCustomDraw;
      ATLASSERT(lpNMLVCD->nmcd.lItemlParam);
      LVCOLORPARAM* pParams = reinterpret_cast<LVCOLORPARAM*>(lpNMLVCD->nmcd.lItemlParam);     
      const LVCOLORPARAM& param = pParams[ lpNMLVCD->iSubItem ];
      if( param.clrText!=-1 ) lpNMLVCD->clrText = param.clrText;
      if( param.clrBackground!=-1 ) lpNMLVCD->clrTextBk = param.clrBackground;
      return CDRF_NEWFONT; // We changed the colors
   }
};

class CColoredListViewCtrl : public CColoredListViewImpl<CColoredListViewCtrl>
{
public:
   DECLARE_WND_SUPERCLASS(_T("WTL_ColoredListView"), GetWndClassName())  
};


/////////////////////////////////////////////////////////////////////////////
// CColoredTreeViewCtrl - A TreeView with item colours

template< class T, class TBase = CTreeViewCtrl, class TWinTraits = CControlWinTraits >
class ATL_NO_VTABLE CColoredTreeViewImpl : 
   public CWindowImpl< T, TBase, TWinTraits >,
   public CCustomDraw< T >
{
public:
   DECLARE_WND_SUPERCLASS(NULL, TBase::GetWndClassName())

   typedef struct tagTVCOLOR {
      COLORREF clrText;
      COLORREF clrBackground;
      COLORREF clrSelText;
      COLORREF clrSelBackground;
      HFONT    hFont;
   } TVCOLOR;
   CSimpleMap< HTREEITEM, TVCOLOR > m_mapColors;

   // Operations

   BOOL SubclassWindow(HWND hWnd)
   {
      ATLASSERT(m_hWnd==NULL);
      ATLASSERT(::IsWindow(hWnd));
#ifdef _DEBUG
      // Check class
      TCHAR szBuffer[20];
      if( ::GetClassName(m_hWnd, szBuffer, (sizeof(szBuffer)/sizeof(TCHAR))-1) ) {
         ATLASSERT(::lstrcmpi(szBuffer, TBase::GetWndClassName())==0);
      }
#endif
      BOOL bRet = CWindowImpl< T, TBase, TWinTraits >::SubclassWindow(hWnd);
      if( bRet ) _Init();
      return bRet;
   }

   // Use syntax:
   //   CColoredTreeViewCtrl::TVCOLOR col;
   //   col.clrText = RGB(120,120,120);
   //   col.clrBackground = -1;
   //   col.clrSelText = -1;
   //   col.clrSelBackground = -1;
   //   col.hFont = NULL;
   //   ctrl.SetItemColors(hItem, &col);
   //
   void SetItemColors(HTREEITEM hItem, TVCOLOR* color)
   {
      ATLASSERT(::IsWindow(m_hWnd));
      m_mapColors.Add(hItem, *color);
   }

   // Implementation

   void _Init()
   {
      ATLASSERT(::IsWindow(m_hWnd));
   }

   // Message map and handlers

   BEGIN_MSG_MAP(CColoredTreeViewImpl)
      MESSAGE_HANDLER(WM_CREATE, OnCreate)
      REFLECTED_NOTIFY_CODE_HANDLER(TVN_DELETEITEM, OnDeleteItem)
      CHAIN_MSG_MAP_ALT( CCustomDraw< T >, 1 )
      DEFAULT_REFLECTION_HANDLER()
   END_MSG_MAP()

   LRESULT OnCreate(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& /*bHandled*/)
   {
      LRESULT lRes = DefWindowProc(uMsg, wParam, lParam);
      _Init();
      return lRes;
   }
   LRESULT OnDeleteItem(int /*idCtrl*/, LPNMHDR pnmh, BOOL& bHandled)
   {
      LPNMTREEVIEW pnmtv = (LPNMTREEVIEW) pnmh;
      m_mapColors.Remove( pnmtv->itemOld.hItem );
      bHandled = FALSE;
      return 0;
   }

   // Custom painting

   DWORD OnPrePaint(int /*idCtrl*/, LPNMCUSTOMDRAW /*lpNMCustomDraw*/)
   {
      return CDRF_NOTIFYITEMDRAW;   // We need per-item notifications
   }
   DWORD OnItemPrePaint(int /*idCtrl*/, LPNMCUSTOMDRAW lpNMCustomDraw)
   {
      LPNMTVCUSTOMDRAW lpNMTVCD = (LPNMTVCUSTOMDRAW) lpNMCustomDraw;
      HTREEITEM hItem = (HTREEITEM) lpNMTVCD->nmcd.dwItemSpec;
      int iIndex = m_mapColors.FindKey( hItem );
      if( iIndex == -1 ) return CDRF_DODEFAULT;
      // The TreeView allows us to change colours for
      // the selected item also (text only).
      const TVCOLOR& col = m_mapColors.GetValueAt(iIndex);
      if( lpNMTVCD->nmcd.uItemState & CDIS_SELECTED ) {
         if( col.clrSelText != (COLORREF) -1 ) lpNMTVCD->clrText = col.clrSelText;
         // Selection background colour cannot be set with current
         // versions of the TreeView control.
         if( col.clrSelBackground != (COLORREF) -1 ) lpNMTVCD->clrTextBk = col.clrSelBackground;
      }
      else {
         if( col.clrText != (COLORREF) -1 ) lpNMTVCD->clrText = col.clrText;
         if( col.clrBackground != (COLORREF) -1 ) lpNMTVCD->clrTextBk = col.clrBackground;
      }
      if( col.hFont != NULL ) ::SelectObject(lpNMTVCD->nmcd.hdc, col.hFont);
      return CDRF_NEWFONT; // We changed the colors/font
   }
};

class CColoredTreeViewCtrl : public CColoredTreeViewImpl<CColoredTreeViewCtrl>
{
public:
   DECLARE_WND_SUPERCLASS(_T("WTL_ColoredTreeView"), GetWndClassName())  
};


#endif // !defined(AFX_COLOREDCONTROLS_H__20020226_0D16_1B29_088A_0080AD509054__INCLUDED_)

