#if !defined(AFX_OFFICE97BUTTONS_H__20020302_2C25_B9FC_3811_0080AD509054__INCLUDED_)
#define AFX_OFFICE97BUTTONS_H__20020302_2C25_B9FC_3811_0080AD509054__INCLUDED_

#pragma once

/////////////////////////////////////////////////////////////////////////////
// Office 97 Buttons
//
// Written by Bjarke Viksoe (bjarke@viksoe.dk)
// Copyright (c) 2002 Bjarke Viksoe.
//
// Add the following macro to the parent's message map:
//   REFLECT_NOTIFICATIONS()
//
// This code may be used in compiled form in any way you desire. This
// file may be redistributed by any means PROVIDING it is 
// not sold for profit without the authors written consent, and 
// providing that this notice and the authors name is included. 
//
// This file is provided "as is" with no expressed or implied warranty.
// The author accepts no liability if it causes any damage to you or your
// computer whatsoever. It's free, so don't hassle me about it.
//
// Beware of bugs.
//

// Include my additional GDI wrappers
#include "atlgdix.h"



/////////////////////////////////////////////////////////////////////////////
// COfficeButtonCtrl - An Office 97 like button

template <class T, class TBase = CButton, class TWinTraits = CControlWinTraits>
class ATL_NO_VTABLE COfficeButtonImpl : 
   public CWindowImpl< T, TBase, TWinTraits >,   
   public CMouseHover< T >
{
public:
   DECLARE_WND_SUPERCLASS(NULL, TBase::GetWndClassName())

   // Color codes for Button control
   COLORREF m_clrText;
   COLORREF m_clrBack;
   // Image support
   CImageList m_ImageList;
   UINT m_nImage;
   UINT m_nSelImage;
   UINT m_nOverImage;
   UINT m_nDisabledImage;

   // Operations

   BOOL SubclassWindow(HWND hWnd)
   {
      ATLASSERT(m_hWnd==NULL);
      ATLASSERT(::IsWindow(hWnd));
#ifdef _DEBUG
      // Check class
      TCHAR szBuffer[16];
      if( ::GetClassName(m_hWnd, szBuffer, sizeof(szBuffer)/sizeof(TCHAR)) ) {
         ATLASSERT(::lstrcmpi(szBuffer, TBase::GetWndClassName())==0);
      }
#endif
      BOOL bRet = CWindowImpl< T, TBase, TWinTraits >::SubclassWindow(hWnd);
      if( bRet ) _Init();
      return bRet;
   }

   void SetTextColor(COLORREF clrText = (COLORREF) -1)
   {
      ATLASSERT(::IsWindow(m_hWnd));
      if( clrText == (COLORREF) -1 ) clrText = ::GetSysColor(COLOR_INFOTEXT);
      m_clrText = clrText;
      // Repaint
      Invalidate();
   }
   void SetBkColor(COLORREF clrBack = (COLORREF) -1)
   {
      ATLASSERT(::IsWindow(m_hWnd));
      if( clrBack == (COLORREF) -1 ) clrBack = ::GetSysColor(COLOR_INFOBK);
      m_clrBack = clrBack;
      // Repaint
      Invalidate();
   }
   void SetImageList(HIMAGELIST hImageList)
   {
      ATLASSERT(::IsWindow(m_hWnd));
      m_ImageList = hImageList;
   }
   void SetImages(UINT nImage, UINT nOverImage = (UINT) -1, UINT nSelImage = (UINT) -1, UINT nDisabledImage = (UINT) -1)
   {
      ATLASSERT(::IsWindow(m_hWnd));
      ATLASSERT(!m_ImageList.IsNull());
      m_nImage = nImage;
      m_nSelImage = nSelImage;
      m_nOverImage = nOverImage;
      m_nDisabledImage = nDisabledImage;
   }

   // Implementation

   void _Init()
   {
      ATLASSERT(::IsWindow(m_hWnd));

      // We need this style to prevent Windows from painting the button
      ModifyStyle(0, BS_OWNERDRAW);
      
      COLORREF clrNone = (COLORREF) -1;
      SetTextColor( clrNone );
      SetBkColor( clrNone );
      m_nImage = m_nSelImage = m_nOverImage = m_nDisabledImage = (UINT) -1;
   }

   // Message map and handlers

   BEGIN_MSG_MAP(COfficeButtonImpl)
      MESSAGE_HANDLER(WM_CREATE, OnCreate)
      MESSAGE_HANDLER(WM_ERASEBKGND, OnEraseBackground)
      MESSAGE_HANDLER(OCM_DRAWITEM, OnDrawItem)
      CHAIN_MSG_MAP( CMouseHover<T> )
   END_MSG_MAP()

   LRESULT OnCreate(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& /*bHandled*/)
   {
      LRESULT lRes = DefWindowProc(uMsg, wParam, lParam);
      _Init();
      return lRes;
   }
   LRESULT OnEraseBackground(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM /*lParam*/, BOOL& /*bHandled*/)
   {
      return 1;   // no background needed
   }
   LRESULT OnDrawItem(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM lParam, BOOL& /*bHandled*/)
   {
      LPDRAWITEMSTRUCT lpDIS = (LPDRAWITEMSTRUCT) lParam;
      ATLASSERT(lpDIS->CtlType==ODT_BUTTON);
      CDCHandle dc = lpDIS->hDC;
      RECT rc = lpDIS->rcItem;
      bool bSelected = (lpDIS->itemState & ODS_SELECTED)!=0;
      bool bDisabled = (lpDIS->itemState & (ODS_DISABLED|ODS_GRAYED))!=0;
      bool bHover = m_fMouseOver;

      // Prepare DC
      dc.SetBkMode(TRANSPARENT);
      HPEN hOldPen = dc.SelectPen( (HPEN) ::GetStockObject(NULL_PEN) );

      // Paint background yellow...
      CBrush brBack;
      brBack.CreateSolidBrush(m_clrBack);
      HBRUSH hOldBrush = dc.SelectBrush(brBack);
      dc.FillSolidRect(&rc, m_clrBack);

      POINT ptArc = { 16, 16 }; // arc size
      RECT rcArc = rc;

      COLORREF clrFrame = RGB(181,178,181);
      COLORREF clrSelBorder = BlendRGB(m_clrBack, m_clrText, 60);
      CPen penSelBorder;
      penSelBorder.CreatePen(PS_SOLID, 1, clrSelBorder);
      CPen penFrame;
      penFrame.CreatePen(PS_SOLID, 1, clrFrame);
      CPen penWhite;
      penWhite.CreatePen(PS_SOLID, 1, RGB(255,255,255));

      if( bHover || bSelected ) {
         // Draw focus shadow
         dc.SelectPen(penFrame);
         dc.RoundRect(&rcArc, ptArc);
         ::InflateRect(&rcArc, 0, -1);
         dc.RoundRect(&rcArc, ptArc);
         ::InflateRect(&rcArc, -1, 0);
      }
      else {
         ::InflateRect(&rcArc, -1, -1);
      }

      // Draw edge
      dc.SelectPen(bHover || bSelected ? penSelBorder : penFrame);
      dc.RoundRect(&rcArc, ptArc);
      ::InflateRect(&rcArc, -1, -1);

      if( bHover || bSelected ) {
         // Create a clipping region so we don't overwrite our
         // cool new round button border...
         CRgn rgn;
         rgn.CreateRoundRectRgn(rcArc.left, rcArc.top, rcArc.right+1, rcArc.bottom+1, ptArc.x, ptArc.y);
         dc.SelectClipRgn(rgn);
         dc.SelectBrush( (HBRUSH) ::GetStockObject(NULL_BRUSH) );

         // The code below should paint the internal button shadows.
         // As you can see the code is pretty crappy.
         // Doesn't work completely either, because the RoundRect() and CreateRoundRectRgn()
         // apparently doesn't create exact regions?!

         const int nAway = 50;

         dc.SelectPen(penWhite);
         dc.RoundRect(rcArc.left-1, rcArc.top, rcArc.right+nAway, rcArc.bottom+nAway, ptArc.x, ptArc.y);
         dc.RoundRect(rcArc.left,   rcArc.top, rcArc.right+nAway, rcArc.bottom+nAway, ptArc.x, ptArc.y);
         dc.RoundRect(rcArc.left+1, rcArc.top, rcArc.right+nAway, rcArc.bottom+nAway, ptArc.x, ptArc.y);
         dc.RoundRect(rcArc.left,   rcArc.top-1, rcArc.right+nAway, rcArc.bottom+nAway, ptArc.x, ptArc.y);
         dc.RoundRect(rcArc.left,   rcArc.top+1, rcArc.right+nAway, rcArc.bottom+nAway, ptArc.x, ptArc.y);
         if( bSelected ) {
            dc.RoundRect(rcArc.left+2, rcArc.top, rcArc.right+nAway, rcArc.bottom+nAway, ptArc.x, ptArc.y);
            dc.RoundRect(rcArc.left+1, rcArc.top+2, rcArc.right+nAway, rcArc.bottom+nAway, ptArc.x, ptArc.y);
         }

         dc.SelectPen(penFrame);
         if( bSelected ) {
            dc.RoundRect(rcArc.left-1, rcArc.top, rcArc.right+nAway, rcArc.bottom+nAway, ptArc.x, ptArc.y);
            dc.RoundRect(rcArc.left,   rcArc.top, rcArc.right+nAway, rcArc.bottom+nAway, ptArc.x, ptArc.y);
         }
         else {
            dc.RoundRect(rcArc.left-nAway, rcArc.top-nAway, rcArc.right,   rcArc.bottom+1, ptArc.x, ptArc.y);
            dc.RoundRect(rcArc.left-nAway, rcArc.top-nAway, rcArc.right+1, rcArc.bottom+1, ptArc.x, ptArc.y);
            dc.RoundRect(rcArc.left-nAway, rcArc.top-nAway, rcArc.right+2, rcArc.bottom+1, ptArc.x, ptArc.y);
            dc.RoundRect(rcArc.left-nAway, rcArc.top-nAway, rcArc.right+1, rcArc.bottom, ptArc.x, ptArc.y);
            dc.RoundRect(rcArc.left-nAway, rcArc.top-nAway, rcArc.right+1, rcArc.bottom+2, ptArc.x, ptArc.y);
         }

         dc.SelectClipRgn(NULL);

      }
      ::InflateRect(&rc, -10, -2 * ::GetSystemMetrics(SM_CYEDGE));

      // Offset when button is selected
      if( bSelected ) ::OffsetRect(&rc, 1,1);

      // Draw image
      UINT nImage = m_nImage;
      if( bHover && m_nOverImage != (UINT) -1 ) nImage = m_nOverImage;
      if( bSelected && m_nSelImage != (UINT) -1 ) nImage = m_nSelImage;
      if( bDisabled && m_nDisabledImage != (UINT) -1 ) nImage = m_nDisabledImage;
      if( !m_ImageList.IsNull() && nImage!=-1 ) {
         SIZE sizeIcon;
         m_ImageList.GetIconSize(sizeIcon);
         POINT pt = { rc.left, rc.top + (((rc.bottom-rc.top)/2) - (sizeIcon.cy/2)) + 1 };
         m_ImageList.Draw(dc, nImage, pt, ILD_TRANSPARENT);
         //rc.left += sizeIcon.cx + 3;
      }

      // Draw text
      UINT nLen = GetWindowTextLength();
      LPTSTR pstr = (LPTSTR) _alloca( (nLen+1)*sizeof(TCHAR) );
      GetWindowText(pstr, nLen+1);
      UINT uFlags = DT_LEFT | DT_SINGLELINE;
      RECT rcText = rc;
      dc.DrawText(pstr, nLen, &rcText, uFlags | DT_CALCRECT);
      ::OffsetRect(&rcText, ((rc.right-rc.left)/2) - ((rcText.right-rcText.left)/2), ((rc.bottom-rc.top)/2) - ((rcText.bottom-rcText.top)/2));
      dc.SetBkMode(TRANSPARENT);
      dc.SetTextColor(bDisabled ? clrFrame : m_clrText);
      dc.DrawText(pstr, nLen, &rcText, uFlags);

      // Draw focus rectangle
      if( lpDIS->itemState & ODS_FOCUS ) {
         ::InflateRect(&rcText, 3, 1);
         // Remove offset when button is selected
         if( bSelected ) ::OffsetRect(&rcText, -1,-1);
         dc.DrawFocusRect(&rcText);
      }

      dc.SelectBrush(hOldBrush);
      dc.SelectPen(hOldPen);

      return TRUE;
   }
};

class COfficeButtonCtrl : public COfficeButtonImpl<COfficeButtonCtrl>
{
public:
   DECLARE_WND_SUPERCLASS(_T("WTL_OfficeButton"), GetWndClassName())  
};


#endif // !defined(AFX_OFFICE97BUTTONS_H__20020302_2C25_B9FC_3811_0080AD509054__INCLUDED_)

