// TreeListCtrl.h: interface for the CTreeListCtrl class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_TREELISTCTRL_H__823C4E6F_CCCB_4AE1_A719_8F832ACF0208__INCLUDED_)
#define AFX_TREELISTCTRL_H__823C4E6F_CCCB_4AE1_A719_8F832ACF0208__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include <vector>
#include "resource.h"

using namespace std;

struct tree_list_info
{
	int				m_nLevel;
	int				m_nImage;
	vector<CString>	m_lstCols;
	bool			m_bCollapsed;
	double			m_data;
};

template<class T, class TBase = CListViewCtrl, class TWinTraits = CControlWinTraits>
class ATL_NO_VTABLE CXTreeListViewImpl : public CWindowImpl<T, TBase, TWinTraits>
{
public:
	DECLARE_WND_SUPERCLASS(NULL,CListViewCtrl::GetWndClassName() );
	BEGIN_MSG_MAP(CXTreeListViewImpl)
		DEFAULT_REFLECTION_HANDLER()
  	END_MSG_MAP()
public:
	void SubclassWindow(HWND hWnd)
	{
		ATLASSERT(m_hWnd == NULL);
		ATLASSERT(::IsWindow(hWnd));
		CWindowImpl< T, CListViewCtrl, TWinTraits >::SubclassWindow(hWnd);
		SetExtendedListViewStyle(LVS_EX_FULLROWSELECT);
		if (m_ilState.IsNull())
			m_ilState.Create(IDB_STATE, 16, 1, RGB(255, 0, 0));
		SetImageList(m_ilState.m_hImageList, LVSIL_STATE);
	}
public:
	CImageList	m_ilState;
	vector<tree_list_info>	m_tree;
public:
	int		AddItem(LPCTSTR lpszItem, int nImage, int nLevel)
	{
		LVITEM			item;
		tree_list_info	info;
		int				nItem;

		nItem = m_tree.size();
		info.m_nLevel	= nLevel;
		info.m_nImage	= nImage;
		info.m_lstCols.resize(GetHeader().GetItemCount());
		info.m_lstCols[0] = lpszItem;
		info.m_bCollapsed = true;
		m_tree.push_back(info);

		item.mask		= LVIF_TEXT | LVIF_IMAGE | LVIF_INDENT | LVIF_STATE | LVIF_PARAM;
		item.iItem		= nItem;
		item.iSubItem	= 0;
		item.pszText	= (LPTSTR)(LPCTSTR)info.m_lstCols[0];
		item.iImage		= info.m_nImage;
		item.iIndent	= info.m_nLevel;
		item.state		= INDEXTOSTATEIMAGEMASK(info.m_bCollapsed ? 1 : 2);
		item.stateMask	= LVIS_STATEIMAGEMASK;
		item.lParam		= nItem;

		if (info.m_nLevel == 0)
		{
			CListViewCtrl::InsertItem(&item);
		}
		return nItem;
	}
	void	SetItemDataEx(int nItem, double dData)
	{
		m_tree.at(nItem).m_data = dData;
	}
	double	GetItemDataEx(int nItem)
	{
		return m_tree.at(nItem).m_data;
	}
	BOOL	SetItemText( int nItem, int nSubItem, LPCTSTR lpszText )
	{
		m_tree.at(nItem).m_lstCols[nSubItem] = lpszText;
		return CListViewCtrl::SetItemText(nItem, nSubItem, lpszText);
	}
	void DeleteItemEx(int nItem)
	{
		vector<tree_list_info> tmp;
		for (size_t i=0; i<m_tree.size(); i++)
			if (i!=nItem) tmp.push_back(m_tree.at(i));
		m_tree.clear();
		for (i=0; i<tmp.size(); i++)
			m_tree.push_back(tmp.at(i));
		tmp.clear();
		CListViewCtrl::DeleteItem(nItem);
	}
	BOOL	DeleteAllItems()
	{
		m_tree.clear();
		return CListViewCtrl::DeleteAllItems();
	}
	int		InsertItem(int nItem, int nPosArray, tree_list_info& info)
	{
		LVITEM			item;

		item.mask		= LVIF_TEXT | LVIF_IMAGE | LVIF_INDENT | LVIF_STATE | LVIF_PARAM;
		item.iItem		= nItem;
		item.iSubItem	= 0;
		item.pszText	= (LPTSTR)(LPCTSTR)info.m_lstCols[0];
		item.iImage		= info.m_nImage;
		item.iIndent	= info.m_nLevel;
		item.state		= INDEXTOSTATEIMAGEMASK(info.m_bCollapsed ? 1 : 2);
		item.stateMask	= LVIS_STATEIMAGEMASK;
		item.lParam		= nPosArray;

		int	nItemRes = CListViewCtrl::InsertItem(&item);

		for (int i = 1; i < info.m_lstCols.size(); i++)
		{
			CListViewCtrl::SetItemText(nItemRes, i, info.m_lstCols[i]);
		}
		return nItemRes;
	}
	void	Collapse(int nPos, int nItem)
	{
		int				nLevel;
		int				nLast;
		LVITEM			item;
		
		tree_list_info&	info = m_tree[nPos];
		info.m_bCollapsed = true;

		SetRedraw(0);
		nLevel	= info.m_nLevel;
		nLast	= GetItemCount();
		item.mask		= LVIF_PARAM;
		item.iSubItem	= 0;

		int nCounter = nItem + 1;
		if (nCounter < nLast)
		{
			do
			{
				item.iItem		= nCounter;
				GetItem(&item);
				if (m_tree[item.lParam].m_nLevel > nLevel)
					DeleteItem(nItem + 1);
			}
			while (m_tree[item.lParam].m_nLevel > nLevel && GetItemCount() > nCounter);
		}
		SetRedraw(1);
	}
	int		Expand(int nItem, bool bRecursive, int& nCounterInsert)
	{
		tree_list_info&	info	= m_tree[nItem];
		info.m_bCollapsed		= false;

		int				nCounter;
		int				nLevel;
		
		nCounter		= nItem + 1;
		nCounterInsert	= nCounterInsert + 1;
		nLevel			= info.m_nLevel;
		while (nCounter < m_tree.size() && m_tree[nCounter].m_nLevel > nLevel)
		{
			if (m_tree[nCounter].m_nLevel > nLevel)
			{
				if (bRecursive)
				{
					m_tree[nCounter].m_bCollapsed = false;
				}
				InsertItem(nCounterInsert, nCounter, m_tree[nCounter]);
			}

			if (bRecursive || !m_tree[nCounter].m_bCollapsed)
			{
				nCounter = Expand(nCounter, bRecursive, nCounterInsert);
			}
			else
			{
				int	nSonLevel;

				nSonLevel = m_tree[nCounter].m_nLevel;
				while (++nCounter < m_tree.size() && m_tree[nCounter].m_nLevel > nSonLevel)
				{
				}

				nCounterInsert++;
			}		
		}
		return nCounter;
	}
};

class CTreeListCtrl : public CXTreeListViewImpl<CTreeListCtrl>
{
public:
   DECLARE_WND_CLASS(_T("WTL_XTreeListView"))
};

#endif // !defined(AFX_TREELISTCTRL_H__823C4E6F_CCCB_4AE1_A719_8F832ACF0208__INCLUDED_)
