#ifndef __ATLCTRLSEXT_H__
#define __ATLCTRLSEXT_H__

/////////////////////////////////////////////////////////////////////////////
// Additional Windows control wrappers
//
// Written by Bjarke Viksoe (bjarke@viksoe.dk)
// Copyright (c) 2002 Bjarke Viksoe.
//
// This code may be used in compiled form in any way you desire. This
// file may be redistributed by any means PROVIDING it is 
// not sold for profit without the authors written consent, and 
// providing that this notice and the authors name is included. 
//
// This file is provided "as is" with no expressed or implied warranty.
// The author accepts no liability if it causes any damage to you or your
// computer whatsoever. It's free, so don't hassle me about it.
//
// Beware of bugs.
//

#pragma once

#ifndef __cplusplus
   #error ATL requires C++ compilation (use a .cpp suffix)
#endif

#ifndef __ATLCTRLS_H__
   #error atlctrlsext.h requires atlctrls.h to be included first
#endif

#if (_WTL_VER < 0x0700)
   #error This file requires WTL version 7.0 or higher
#endif


namespace WTL
{

/////////////////////////////////////////////////////////////////////////////
// A mananged ImageList control

class CImageListCtrl : public CImageList
{
public:
   ~CImageListCtrl()
   {
      Destroy();
   }
};

typedef CImageList CImageListHandle;


/////////////////////////////////////////////////////////////////////////////
// CCustomXXXDraw - MI class for Common Controls custom-draw support

// NOTE: WTL 7 includes a fine CCustomDraw class, but my class supports
//       the correct NM_CUSTOMDRAW structures for each control type.
template< class T, typename TNMCUSTOMDRAW >
class CCustomDrawEx
{
public:
   typedef CCustomDrawEx< T, TNMCUSTOMDRAW > thisClass;

#if (_ATL_VER < 0x0700)
   BOOL m_bHandledCD;

   BOOL IsMsgHandled() const
   {
      return m_bHandledCD;
   }
   void SetMsgHandled(BOOL bHandled)
   {
      m_bHandledCD = bHandled;
   }
#endif // _ATL_VER

   // Message map and handlers
   
   BEGIN_MSG_MAP( thisClass )
      NOTIFY_CODE_HANDLER(NM_CUSTOMDRAW, OnCustomDraw)
   ALT_MSG_MAP(1)
      REFLECTED_NOTIFY_CODE_HANDLER(NM_CUSTOMDRAW, OnCustomDraw)
   END_MSG_MAP()

   LRESULT OnCustomDraw(int idCtrl, LPNMHDR pnmh, BOOL& bHandled)
   {
      T* pT = static_cast<T*>(this);
      pT->SetMsgHandled(TRUE);
      TNMCUSTOMDRAW lpNM = reinterpret_cast<TNMCUSTOMDRAW> pnmh;
      DWORD dwRet = 0;
      switch( lpNM->nmcd.dwDrawStage ) {
      case CDDS_PREPAINT:
         dwRet = pT->OnPrePaint(idCtrl, lpNM);
         break;
      case CDDS_POSTPAINT:
         dwRet = pT->OnPostPaint(idCtrl, lpNM);
         break;
      case CDDS_PREERASE:
         dwRet = pT->OnPreErase(idCtrl, lpNM);
         break;
      case CDDS_POSTERASE:
         dwRet = pT->OnPostErase(idCtrl, lpNM);
         break;
      case CDDS_ITEMPREPAINT:
         dwRet = pT->OnItemPrePaint(idCtrl, lpNM);
         break;
      case CDDS_ITEMPOSTPAINT:
         dwRet = pT->OnItemPostPaint(idCtrl, lpNM);
         break;
#if (_WIN32_IE >= 0x0400)
      case CDDS_ITEMPREPAINT|CDDS_SUBITEM:
         dwRet = pT->OnSubItemPrePaint(idCtrl, lpNM);
         break;
      case CDDS_ITEMPOSTPAINT|CDDS_SUBITEM:
         dwRet = pT->OnSubItemPostPaint(idCtrl, lpNM);
         break;
#endif // _WIN32_IE
      case CDDS_ITEMPREERASE:
         dwRet = pT->OnItemPreErase(idCtrl, lpNM);
         break;
      case CDDS_ITEMPOSTERASE:
         dwRet = pT->OnItemPostErase(idCtrl, lpNM);
         break;
      default:
         pT->SetMsgHandled(FALSE);
         break;
      }
      bHandled = pT->IsMsgHandled();
      return dwRet;
   }

   // Overrideables

   DWORD OnPrePaint(int /*idCtrl*/, TNMCUSTOMDRAW /*lpNM*/)
   {
      return CDRF_DODEFAULT;
   }
   DWORD OnPostPaint(int /*idCtrl*/, TNMCUSTOMDRAW /*lpNM*/)
   {
      return CDRF_DODEFAULT;
   }
   DWORD OnPreErase(int /*idCtrl*/, TNMCUSTOMDRAW /*lpNM*/)
   {
      return CDRF_DODEFAULT;
   }
   DWORD OnPostErase(int /*idCtrl*/, TNMCUSTOMDRAW /*lpNM*/)
   {
      return CDRF_DODEFAULT;
   }
   DWORD OnItemPrePaint(int /*idCtrl*/, TNMCUSTOMDRAW /*lpNM*/)
   {
      return CDRF_DODEFAULT;
   }
   DWORD OnItemPostPaint(int /*idCtrl*/, TNMCUSTOMDRAW /*lpNM*/)
   {
      return CDRF_DODEFAULT;
   }
#if (_WIN32_IE >= 0x0400)
   DWORD OnSubItemPrePaint(int /*idCtrl*/, TNMCUSTOMDRAW /*lpNM*/)
   {
      return CDRF_DODEFAULT;
   }
   DWORD OnSubItemPostPaint(int /*idCtrl*/, TNMCUSTOMDRAW /*lpNM*/)
   {
      return CDRF_DODEFAULT;
   }
#endif // _WIN32_IE
   DWORD OnItemPreErase(int /*idCtrl*/, TNMCUSTOMDRAW /*lpNM*/)
   {
      return CDRF_DODEFAULT;
   }
   DWORD OnItemPostErase(int /*idCtrl*/, TNMCUSTOMDRAW /*lpNM*/)
   {
      return CDRF_DODEFAULT;
   }
};

template< class T >
class CCustomTreeViewDraw : public CCustomDrawEx<T, LPNMTVCUSTOMDRAW> { };

template< class T >
class CCustomListViewDraw : public CCustomDrawEx<T, LPNMCUSTOMDRAW> { };

template< class T >
class CCustomToolBarDraw : public CCustomDrawEx<T, LPNMTBCUSTOMDRAW> { };

template< class T >
class CCustomToolTipDraw : public CCustomDrawEx<T, LPNMTTCUSTOMDRAW> { };


/////////////////////////////////////////////////////////////////////////////
// Customizable toolbar

// Simple implementation of ToolBar customization support.
// It handles only one ToolBar in total!
// Let the parent window derive from the class; chain it in the message map.
// Call InitToolbar() after creating the ToolBar control.
template< class T >
class CCustomizableToolBarCommands
{
public:
   typedef CCustomizableToolBarCommands<T> thisClass;

   CSimpleArray<TBBUTTON> m_aButtons;

   // Operations

   BOOL InitToolBar(HWND hWndToolBar, UINT nResource, BOOL bInitialSeparator = FALSE)
   {
      ATLASSERT(::IsWindow(hWndToolBar));

      // The ToolBar is adjustable!
      CToolBarCtrl tb = hWndToolBar;
      tb.ModifyStyle(0, CCS_ADJUSTABLE);

      // Gather information about toolbar buttons by building the toolbar.
      // Needed so we can reset it later on.
      // This code is almost identical to the CFrameWindowImplBase::CreateSimpleToolBarCtrl
      // code, but it also needs to build it the exact same way...
      HINSTANCE hInst = _Module.GetResourceInstance();
      HRSRC hRsrc = ::FindResource(hInst, (LPCTSTR) nResource, RT_TOOLBAR);
      if (hRsrc == NULL) return FALSE;
      HGLOBAL hGlobal = ::LoadResource(hInst, hRsrc);
      if( hGlobal == NULL ) return FALSE;

      struct _AtlToolBarData
      {
         WORD wVersion;
         WORD wWidth;
         WORD wHeight;
         WORD wItemCount;
         //WORD aItems[wItemCount]
         WORD* items() { return (WORD*)(this+1); }
      };
      _AtlToolBarData* pData = (_AtlToolBarData*) ::LockResource(hGlobal);
      if( pData==NULL ) return FALSE;
      ATLASSERT(pData->wVersion==1);

      WORD* pItems = pData->items();
      // Set initial separator (half width)
      if( bInitialSeparator ) {
         TBBUTTON bt;
         bt.iBitmap = 4;
         bt.idCommand = 0;
         bt.fsState = 0;
         bt.fsStyle = TBSTYLE_SEP;
         bt.dwData = 0;
         bt.iString = 0;
         m_aButtons.Add(bt);
      }
      // Scan other buttons
      int nBmp = 0;
      for( int i=0, j= bInitialSeparator ? 1 : 0; i<pData->wItemCount; i++, j++ ) {
         if( pItems[i] != 0 ) {
            TBBUTTON bt;
            bt.iBitmap = nBmp++;
            bt.idCommand = pItems[i];
            bt.fsState = TBSTATE_ENABLED;
            bt.fsStyle = TBSTYLE_BUTTON;
            bt.dwData = 0;
            bt.iString = 0;
            m_aButtons.Add(bt);
         }
         else {
            TBBUTTON bt;
            bt.iBitmap = 8;
            bt.idCommand = 0;
            bt.fsState = 0;
            bt.fsStyle = TBSTYLE_SEP;
            bt.dwData = 0;
            bt.iString = 0;
            m_aButtons.Add(bt);
         }
      }
      return TRUE;
   }

   // Message map and handler

   BEGIN_MSG_MAP( thisClass )
#if (_WIN32_IE >= 0x0500)
      NOTIFY_CODE_HANDLER(TBN_INITCUSTOMIZE, OnTbInitCustomize)
      NOTIFY_CODE_HANDLER(TBN_SAVE, OnTbSave)
      NOTIFY_CODE_HANDLER(TBN_RESTORE, OnTbRestore)
#endif
      NOTIFY_CODE_HANDLER(TBN_BEGINADJUST, OnTbBeginAdjust)
      NOTIFY_CODE_HANDLER(TBN_ENDADJUST, OnTbEndAdjust)
      NOTIFY_CODE_HANDLER(TBN_RESET, OnTbReset)
      NOTIFY_CODE_HANDLER(TBN_TOOLBARCHANGE, OnTbToolBarChange)
      NOTIFY_CODE_HANDLER(TBN_QUERYINSERT, OnTbQueryInsert)
      NOTIFY_CODE_HANDLER(TBN_QUERYDELETE, OnTbQueryDelete)
      NOTIFY_CODE_HANDLER(TBN_GETBUTTONINFO, OnTbGetButtonInfo)      
   END_MSG_MAP()

#if (_WIN32_IE >= 0x0500)
   LRESULT OnTbInitCustomize(int /*idCtrl*/, LPNMHDR /*pnmh*/, BOOL& /*bHandled*/)
   {
      // We don't really know how to help, so hide the HELP button!
      return TBNRF_HIDEHELP;
   }
   LRESULT OnTbSave(int /*idCtrl*/, LPNMHDR pnmh, BOOL& /*bHandled*/)
   {
      CToolBarCtrl tb = pnmh->hwndFrom;
      LPNMTBSAVE pnmtbr = (LPNMTBSAVE) pnmh;
      if( pnmtbr->iItem==-1 ) return 0;
      // TODO: Add code here
      return 0;
   }
   LRESULT OnTbRestore(int /*idCtrl*/, LPNMHDR pnmh, BOOL& /*bHandled*/)
   {
      CToolBarCtrl tb = pnmh->hwndFrom;
      LPNMTBRESTORE pnmtbr = (LPNMTBRESTORE) pnmh;
      if( pnmtbr->iItem==-1 ) return 0;
      // TODO: Add code here
      return 0;
   }
#endif

   LRESULT OnTbBeginAdjust(int /*idCtrl*/, LPNMHDR /*pnmh*/, BOOL& /*bHandled*/)
   {
      ATLASSERT(m_aButtons.GetSize()>0); // Remember to call InitToolBar()!
      return TRUE;
   }
   LRESULT OnTbReset(int /*idCtrl*/, LPNMHDR pnmh, BOOL& /*bHandled*/)
   {
      LPTBNOTIFY lpTbNotify = (LPTBNOTIFY) pnmh;
      // Restore the old button-set
      CToolBarCtrl tb = lpTbNotify->hdr.hwndFrom;
      while( tb.GetButtonCount() > 0 ) tb.DeleteButton(0);
      tb.AddButtons(m_aButtons.GetSize(), m_aButtons.GetData());
      return TRUE;
   }
   LRESULT OnTbEndAdjust(int /*idCtrl*/, LPNMHDR /*pnmh*/, BOOL& /*bHandled*/)
   {
      return TRUE;
   }
   LRESULT OnTbToolBarChange(int /*idCtrl*/, LPNMHDR /*pnmh*/, BOOL& bHandled)
   {
      bHandled = FALSE;
      return 0;
   }
   LRESULT OnTbQueryInsert(int /*idCtrl*/, LPNMHDR /*pnmh*/, BOOL& /*bHandled*/)
   {
      // Allow all buttons!
      return TRUE;
   }
   LRESULT OnTbQueryDelete(int /*idCtrl*/, LPNMHDR /*pnmh*/, BOOL& /*bHandled*/)
   {
      // All buttons can be deleted!
      return TRUE;
   }
   LRESULT OnTbGetButtonInfo(int /*idCtrl*/, LPNMHDR pnmh, BOOL& /*bHandled*/)
   {
      LPTBNOTIFY lpTbNotify = (LPTBNOTIFY) pnmh;
      CToolBarCtrl tb = lpTbNotify->hdr.hwndFrom;
      // The toolbar requests information about buttons that we don't know of...
      if( lpTbNotify->iItem >= m_aButtons.GetSize() ) return FALSE;
      // Locate tooltip text and copy second half of it.
      // This is the same code as CFrameWindowImplBase uses, despite how 
      // dangerous it may look...
      TCHAR szBuff[256] = { 0 };
      LPCTSTR pstr = szBuff;
#if (_ATL_VER < 0x0700)
      int nRet = ::LoadString(_Module.GetResourceInstance(), m_aButtons[lpTbNotify->iItem].idCommand, szBuff, 255);
#else
      int nRet = ATL::AtlLoadString(m_aButtons[lpTbNotify->iItem].idCommand, szBuff, 255);
#endif
      for( int i = 0; i < nRet; i++ ) {
         if( szBuff[i] == _T('\n') ) {
            pstr = szBuff + i + 1;
            break;
         }
      }
      lpTbNotify->tbButton = m_aButtons[lpTbNotify->iItem];
      ::lstrcpyn(lpTbNotify->pszText, pstr, lpTbNotify->cchText);
      lpTbNotify->cchText = ::lstrlen(pstr);
      return TRUE;
   }
};


/////////////////////////////////////////////////////////////////////////////
// ActiveX helpers

#ifdef __ATLHOST_H__

// Load HTML into MSHTML ActiveX control
inline BOOL AtlLoadHTML(IUnknown* pUnkControl, LPCSTR pstrHTML)
{
   ATLASSERT(!::IsBadStringPtrA(pstrHTML, -1));
   HANDLE hHTMLText = ::GlobalAlloc( GPTR, (::lstrlenA(pstrHTML)+1) * sizeof(CHAR) );
   if( hHTMLText==NULL ) return FALSE;
   IStream* pStream;
   ::lstrcpyA( (CHAR*) hHTMLText, pstrHTML );
   HRESULT Hr = ::CreateStreamOnHGlobal(hHTMLText, TRUE, &pStream);
   if( SUCCEEDED(Hr) ) {
      IPersistStreamInit* pPersistStreamInit;
      Hr = pUnkControl->QueryInterface(IID_IPersistStreamInit, (LPVOID*) &pPersistStreamInit);
      if( SUCCEEDED(Hr) ) {
           Hr = pPersistStreamInit->InitNew();
           if( SUCCEEDED(Hr) ) {
               Hr = pPersistStreamInit->Load(pStream);
           }
      }
      pPersistStreamInit->Release();
   }
   pStream->Release();
   return SUCCEEDED(Hr);
};

#endif // __ATLHOST_H__


}; // namespace WTL

#endif // __ATLCTRLSEXT_H__
